/* Petter Schau 1996                                 */
/* Initialization of the M68000 emulation in Fellow  */
/* Currently only 68000 emulation                    */
/* Linux portability problem: sprintf */


#include <stdio.h>
#include "defs.h"
#include "memory.h"
#include "inout.h"
#include "chip.h"
#include "68000ini.h"

/* Registers and tables */

ULO t[65536][8],pc,usp,ssp,cyclecount,lastcyclecount,thiscycle;
ULO sr;
UBY bcd2hex[256],hex2bcd[256];
UBY mulutab[256],mulstab[256];

ULO config_cpu_speed;
ULO config_cpu_type;


#ifdef PREFETCH

ULO prefetch_word;

#endif

ULO interruptcyclesleft = 0;
ULO interruptlevel=0;
ULO interruptaddress=0;
ULO interruptflag=0;
ULO cpustopflag=0;

ULO exceptionstack;
ULO exceptionbackdooraddress;

ULO int2int[16] = {1,1,1,2,3,3,3,4,4,4,4,5,5,6,6};

/* Truth table Variables  True when 0X0X or 1X1X */
UBY ccctt[16] = {1,1,0,0,1,1,0,0,0,0,1,1,0,0,1,1};

/* Truth table Variables  True when 0X1X or 1X0X */
UBY ccdtt[16] = {0,0,1,1,0,0,1,1,1,1,0,0,1,1,0,0};

/* Truth table Variables  True when 000X or 111X */

UBY ccett[16] = {1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,1};

/* Truth table Variables  True when X1XX 1X0X or 0X1X */

UBY ccftt[16] = {0,0,1,1,1,1,1,1,1,1,0,0,1,1,1,1};


/* Tables of pointers to ea calculation routines        */
/* These are used when initializing the opcode tables   */
/* and must be initialized first with either x00 or x20 */
/* versions                                             */

eareadfunc arb[12],arw[12],arl[12],parb[12],parw[12],parl[12];
eawritefunc awb[12],aww[12],awl[12];
eacalcfunc eac[12];

/* Tables of pointers to ea calculation routines */
/* These are for 68000 emulation opcode tables   */

eareadfunc arb68000[12] = {arb00,NULL,arb02,arb03,arb04,arb05,arb06,
				 arb70,arb71,arb72,arb73,arb74};
eareadfunc arw68000[12] = {arw00,arw01,arw02,arw03,arw04,arw05,arw06,
				 arw70,arw71,arw72,arw73,arw74};
eareadfunc arl68000[12] = {arl00,arl01,arl02,arl03,arl04,arl05,arl06,
				 arl70,arl71,arl72,arl73,arl74};
eareadfunc parb68000[12] = {arb00,NULL,arb02,parb03,parb04,parb05,
				  parb06,parb70,parb71,parb72,parb73,parb74};
eareadfunc parw68000[12] = {arw00,arw01,arw02,parw03,parw04,parw05,
				  parw06,parw70,parw71,parw72,parw73,parw74};
eareadfunc parl68000[12] = {arl00,arl01,arl02,parl03,parl04,parl05,
				  parl06,parl70,parl71,parl72,parl73,parl74};
eawritefunc awb68000[12] = {awb00,NULL,awb02,awb03,awb04,awb05,awb06,
				  awb70,awb71};
eawritefunc aww68000[12] = {aww00,aww01,aww02,aww03,aww04,aww05,aww06,
				  aww70,aww71};
eawritefunc awl68000[12] = {awl00,awl01,awl02,awl03,awl04,awl05,awl06,
				  awl70,awl71};
eacalcfunc eac68000[12] = {NULL,NULL,eac02,NULL,NULL,eac05,eac06,eac70,
				 eac71,eac72,eac73,NULL};


/* Tables for cycle count calculation for each addressmode */
/* Must be initialized for either x00 or x20               */

UBY tarb[12];
UBY tarw[12];
UBY tarl[12];

/* Tables for address mode cycle counts, 68000 */

const UBY tarb68000[12] = {0,0,4,4, 6, 8,10, 8,12, 8,10,4};
const UBY tarw68000[12] = {0,0,4,4, 6, 8,10, 8,12, 8,10,4};
const UBY tarl68000[12] = {0,0,8,8,10,12,14,12,16,12,14,8};

/* Address modes included in the different classes */

const UBY allmodes[16]  = {1,1,1,1, 1, 1, 1, 1, 1, 1, 1,1,0,0,0,0};
const UBY data[16]      = {1,0,1,1, 1, 1, 1, 1, 1, 1, 1,1,0,0,0,0};
const UBY memory[16]    = {0,0,1,1, 1, 1, 1, 1, 1, 1, 1,1,0,0,0,0};
const UBY control[16]   = {0,0,1,0, 0, 1, 1, 1, 1, 1, 1,0,0,0,0,0};
const UBY alterable[16] = {1,1,1,1, 1, 1, 1, 1, 1, 0, 0,0,0,0,0,0};

ULO statustab[4096];

/* Cpu emulation initialization */

void makebcdtabs(void) {
  ULO x;
  for (x = 0; x < 100; x++) hex2bcd[x] = x % 10 | (x / 10) << 4;
  for (x = 100; x < 256; x++) hex2bcd[x] = 0;
  for (x = 0; x < 256; x++)
    bcd2hex[x] = (((x&0xf)<10)&&((x&0xf0)<0xa0)) ? (x&0xf)+((x&0xf0)>>4)*10:0;
}


/* Disassembly of the different address-modes.
  Parameters:
   ULO reg  - The register used in the address-mode
   ULO pcp  - The address of the next byte not used.
   char *st - The string to write the dissassembly to.
   ULO *pos - The position in the string where the hex-words used are written
  Returnvalue:
   The address of the next byte not used. (PC after the modedata) */

ULO dis00(ULO reg,ULO pcp,char *st,ULO *pos) {
  sprintf(st,"%sD%1d",st,reg);
  return pcp;
}

ULO dis01(ULO reg,ULO pcp,char *st,ULO *pos) {
  sprintf(st,"%sA%1d",st,reg);
  return pcp;
}

ULO dis02(ULO reg,ULO pcp,char *st,ULO *pos) {
  sprintf(st,"%s(A%1d)",st,reg);
  return pcp;
}

ULO dis03(ULO reg,ULO pcp,char *st,ULO *pos) {
  sprintf(st,"%s(A%1d)+",st,reg);
  return pcp;
}

ULO dis04(ULO reg,ULO pcp,char *st,ULO *pos) {
  sprintf(st,"%s-(A%1d)",st,reg);
  return pcp;
}

ULO dis05(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);
  sprintf(st,"%s$%.4X(A%1d)",st,j,reg);
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO dis06(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);

  sprintf(st,"%s$%.2X(A%1d,%c%1d.%c)",st,j&0xff,reg,(j&0x8000) ? 'A':'D',(j&0x7000)>>12, (j&0x0800) ? 'L':'W');
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO dis70(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);
  sprintf(st,"%s$%.4X.W",st,j);
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO dis71(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetl(pcp);
  sprintf(st,"%s$%.8X.L",st,j);
  sprintf(&st[*pos],"%.8X",j);
  *pos += 9;
  st[(*pos)-1] = ' '; 
  return pcp+4;
}

ULO dis72(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);
  sprintf(st,"%s$%.4X(PC)",st,j);
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO dis73(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);

  sprintf(st,"%s$%.2X(A%1d,%c%1d.%c)",st,j&0xff,reg,(j&0x8000) ? 'A':'D',(j&0x7000)>>12, (j&0x0800) ? 'L':'W');
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO disb74(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);
  sprintf(st,"%s#$%.2X",st,j&0x00ff);
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO disw74(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetw(pcp);
  sprintf(st,"%s#$%.4X",st,j);
  sprintf(&st[*pos],"%.4X",j);
  *pos += 5;
  st[(*pos)-1] = ' '; 
  return pcp+2;
}

ULO disl74(ULO reg,ULO pcp,char *st,ULO *pos) {
  ULO j = fetl(pcp);
  sprintf(st,"%s#$%.8X",st,j);
  sprintf(&st[*pos],"%.8X",j);
  *pos += 9;
  st[(*pos)-1] = ' '; 
  return pcp+4;
}


ULO disfordel(ULO reg,ULO pcp,char *st,ULO size,ULO mode,ULO *pos) {
  switch (mode) {
    case 0:  return dis00(reg,pcp,st,pos);
    case 1:  return dis01(reg,pcp,st,pos);
    case 2:  return dis02(reg,pcp,st,pos);
    case 3:  return dis03(reg,pcp,st,pos);
    case 4:  return dis04(reg,pcp,st,pos);
    case 5:  return dis05(reg,pcp,st,pos);
    case 6:  return dis06(reg,pcp,st,pos);
    case 7:  return dis70(reg,pcp,st,pos);
    case 8:  return dis71(reg,pcp,st,pos);
    case 9:  return dis72(reg,pcp,st,pos);
    case 10: return dis73(reg,pcp,st,pos);
    case 11: return (size==8)  ? disb74(reg,pcp,st,pos):
                    (size==16) ? disw74(reg,pcp,st,pos):
                                 disl74(reg,pcp,st,pos);
    default: return pcp;
    }
}


ULO size000110(ULO s) { return (s == 0x00) ? 8: (s == 0x40) ? 16:32;}


/*  Illegal instruction */

ULO i00dis(ULO prc,ULO opc,char *st) {
  sprintf(st,"$%.6X %.4X                   ILLEGAL INSTRUCTION",prc,opc);
  return prc+2;
}

/*  Instruction ABCD.B Ry,Rx / ABCD.B -(Ay),-(Ax)
    Table-usage: 0 - Routinepointer 1 - disasm routine 2 - Rx*4(dest) 3 - Ry*4
*/

ULO i01dis(ULO prc,ULO opc,char *st) {
  sprintf(st,((opc&0x8 == 0) ?
                "$%.6X %.4X                   ABCD.B  D%1d,D%1d":
                "$%.6X %.4X                   ABCD.B  -(A%1d),-(A%1d)"),
                prc,opc,opc&0x3,(opc&0xe00)>>9);
  return prc+2;
}

void i01ini(void) {
  ULO op=0xc100,ind,rm,rx,ry;
  for (rm = 0; rm <= 1; rm++) 
    for (rx = 0; rx <= 7; rx++) 
      for (ry = 0; ry <= 7; ry++) {
        ind = op|ry|rx<<9|rm<<3;
        t[ind][0] = (rm == 0) ?
                       (ULO) i01_1:
                       (ULO) i01_2;
        t[ind][1] = (ULO) i01dis;
        t[ind][2] = (rm == 0) ?
                       (rx*4 + (ULO) d):
                       (rx*4 + (ULO) a);
        t[ind][3] = (rm == 0) ?
                       (ry*4 + (ULO) d):
                       (ry*4 + (ULO) a);
        }
}


/*
 Instruction ADD.X <ea>,Dn / ADD.X Dn,<ea>
 Table-usage:
 Type 1:  add <ea>,Dn
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5-cycle count                   
 Type 2:  add Dn,<ea>
  0-Routinepointer 1-disasm routine  2-<ea>reg 3-get <ea> routine
                   4-Dreg        5-cycle count   6-write<ea> routine                
*/

ULO i02dis(ULO prc,ULO opc,char *st) {
  ULO pos=13,reg=opc&SREG,dreg=(opc&DREG)>>9,o=opc&0x100,mode=(opc&SMOD)>>3,
      size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   ADD.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = (o == 0) ? disfordel(reg,prc+2,st,size,mode,&pos) :
                   disfordel(dreg,prc+2,st,size,0,&pos);
  strcat(st,",");
  prc = (o != 0) ? disfordel(reg,prc,st,size,mode,&pos) :
                   disfordel(dreg,prc,st,size,0,&pos);
  return prc;
}

void i02ini(void)
{
  ULO op=0xd000,ind,regd,modes,regs,flats,size,o;
  for (o = 0; o < 2; o++)
    for (size = 0; size < 3; size++)
      for (regd = 0; regd < 8; regd++) 
        for (modes = 0; modes < 8; modes++) 
          for (regs = 0; regs < 8; regs++) {
            flats = (modes != 7) ?
                       modes:
                       (modes+regs);
            if ((o==0 && size == 0 && data[flats]) ||
                (o==0 && size != 0 && allmodes[flats]) ||
                (o==1 && alterable[flats] && memory[flats])) {
              ind = op|regd<<9|o<<8|size<<6|modes<<3|regs;
              t[ind][0] = (o == 0) ?
                             (size == 0) ?
                                (ULO) i02_B_1:
                                (size == 1) ?
                                   (ULO) i02_W_1:
                                   (ULO) i02_L_1:
                             (size == 0) ?
                                (ULO) i02_B_2:
                                (size == 1) ?
                                   (ULO) i02_W_2:
                                   (ULO) i02_L_2;
              t[ind][1] = (ULO) i02dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO) ((o == 0) ?
                             (size == 0) ?
                                arb[flats]:
                                (size == 1) ?
                                   arw[flats]:
                                   arl[flats]:
                             (size == 0) ?
                                parb[flats]:
                                (size == 1) ?
                                   parw[flats]:
                                   parl[flats]);
              t[ind][4] = regd*4 + (ULO) d;
              t[ind][5] = (o == 0) ?
                             (size == 0) ?
                                (4+tarb[flats]):
                                (size == 1) ?
                                   (4+tarw[flats]):
                                   (flats <= 1 || flats == 11) ?
                                      (8+tarl[flats]):
                                      (6+tarl[flats]):
                             (size == 0) ?
                                (8 + tarb[flats]):
                                (size == 1) ?
                                   (8 + tarw[flats]):
                                   (12 + tarl[flats]);
              if (o == 1) t[ind][6] = (ULO) ((size == 0) ?
                                         awb[flats]:
                                         (size == 1) ?
                                            aww[flats]:
                                            awl[flats]);
              }
            }
}


/*
 Instruction ADDA.W/L <ea>,An
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Areg        5-cycle count                   
*/



ULO i03dis(ULO prc,ULO opc,char *st) {
  ULO pos=13,reg=opc&SREG,dreg=(opc&DREG)>>9,o=opc&0x100,mode=(opc&SMOD)>>3,
      size;
  size = (o == 0) ?
            16:
            32;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   ADDA.   ",prc,opc);
  st[36] = (size == 16) ?
              'W':
              'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,size,1,&pos);
  return prc;
}

void i03ini(void)
{
  ULO op=0xd0c0,ind,regd,modes,regs,flats,o;
  for (o = 0; o < 2; o++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) 
          if (allmodes[(flats = (modes != 7) ? modes : modes+regs)]) {
            ind = op|regd<<9|o<<8|modes<<3|regs;
            t[ind][0] = (o == 0) ?
                           (ULO) i03_W:
                           (ULO) i03_L;
            t[ind][1] = (ULO) i03dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO) ((o == 0) ?
                           arw[flats]:
                           arl[flats]);
            t[ind][4] = regd*4 + (ULO) a;
            t[ind][5] = (o==0) ?
                           (8 + tarw[flats]):
                           (flats <= 1 || flats == 11) ?
                              (8 + tarl[flats]):
                              (6 + tarl[flats]);
            }
}


/*
 Instruction ADDI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine               
*/

ULO i04dis(ULO prc,ULO opc,char *st) 
{
  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   ADDI.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  prc = disfordel(reg,prc,st,size,mode,&pos);
  return prc;
}

void i04ini(void)
{
  ULO op=0x0600,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : modes+regs;
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         (ULO) i04_B:
                         (size == 1) ?
                            (ULO) i04_W:
                            (ULO) i04_L; 
          t[ind][1] = (ULO) i04dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) ((size == 0) ?
                         parb[flats]:
                         (size == 1) ?
                            parw[flats]:
                            parl[flats]);
          t[ind][4] = (size == 0) ?
                         (flats == 0) ?
                            8:
                            (12 + tarb[flats]):
                         (size == 1)  ?
                            (flats == 0) ?
                               8:
                               (12 + tarw[flats]):
                            (flats == 0) ?
                               16:
                               (20 + tarl[flats]);
          t[ind][5] = (ULO) ((size == 0) ?
                         awb[flats]:
                         (size == 1) ? 
                            aww[flats]:
                            awl[flats]);
          }
        }
}

/*
 Instruction ADDQ.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine 6-add immediate              
*/

ULO i05dis(ULO prc,ULO opc,char *st) 
{
  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   ADDQ.   #$%.1d,",prc,opc,t[opc][6]);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  return prc;
}


void i05ini(void)
{
  ULO op=0x5000,ind,imm,modes,regs,flats,size,o;
    for (size = 0; size < 3; size++)
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) 
          for (imm = 0; imm < 8; imm++) {
            flats = (modes != 7) ?
                       modes:
                       modes+regs;
            if ((size == 0 && alterable[flats] && modes != 1) ||
                (size != 0 && alterable[flats])) {
              ind = op|imm<<9|size<<6|modes<<3|regs;
              if (modes == 1 && (size > 0)) t[ind][0] = (ULO) i05_ADR;
              else
                t[ind][0] =
                          (size == 0) ?
                             (ULO) i05_B:
                             (size == 1) ?
                                (ULO) i05_W:
                                (ULO) i05_L;
              t[ind][1] = (ULO) i05dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO)((size == 0) ?
                             parb[flats]:
                             (size == 1) ?
                                parw[flats]:
                                parl[flats]);
              t[ind][4] = (size == 0) ?
                             (flats == 0) ?
                                (4+tarb[flats]):
                                (8+tarb[flats]):
                             (size == 1) ?
                                (flats == 0) ?
                                   (4+tarw[flats]):
                                   (8+tarw[flats]):
                                (flats < 2) ?
                                   (8+tarl[flats]):
                                   (12+tarl[flats]);
              t[ind][5] = (ULO)((size == 0) ?
                             awb[flats]:
                             (size == 1) ?
                                aww[flats]:
                                awl[flats]);
              t[ind][6] = (imm == 0) ?
                             8:
                             imm;
              }
            }
}

/*
 Instruction ADDX.X Ry,Rx / ADD.X -(Ay),-(Ax)
 Table-usage: 0 - Routinepointer 1 - disasm routine 2 - Rx*4(dest) 3 - Ry*4
*/

ULO i06dis(ULO prc,ULO opc,char *st)
{
  ULO size=(opc&0x00c0)>>6;
  sprintf(st,(((opc&0x8) == 0) ?
                "$%.6X %.4X                   ADDX.%s  D%d,D%d":
                "$%.6X %.4X                   ADDX.%s  -(A%d),-(A%d)"),
                prc,opc,(size == 0) ? "B":((size == 1) ? "W":"L"),(opc&SREG),(opc&DREG)>>9);
  return prc+2;
}

void i06ini(void)
{
  ULO op=0xd100,ind,rm,rx,ry,sz;
  for (sz = 0; sz <= 2; sz++)
    for (rm = 0; rm <= 1; rm++)
      for (rx = 0; rx <= 7; rx++) 
        for (ry = 0; ry <= 7; ry++) {
          ind = op|ry|rx<<9|rm<<3|sz<<6;
          t[ind][0] = (rm == 0) ?
                       ((sz == 0) ?
                          (ULO) i06_B_1:
                          (sz == 1) ?
                             (ULO) i06_W_1:
                             (ULO) i06_L_1):
                       ((sz == 0) ?
                          (ULO) i06_B_2:
                          (sz == 1) ?
                             (ULO) i06_W_2:
                             (ULO) i06_L_2);
        t[ind][1] = (ULO) i06dis;
        t[ind][2] = (rm == 0) ?
                       (rx*4 + (ULO) d):
                       (rx*4 + (ULO) a);
        t[ind][3] = (rm == 0) ?
                       (ry*4 + (ULO) d):
                       (ry*4 + (ULO) a);
        }
}


/*
 Instruction AND.X <ea>,Dn / AND.X Dn,<ea>
 Table-usage:
 Type 1:  and <ea>,Dn

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5-cycle count                   
 Type 2:  and Dn,<ea>

  0-Routinepointer 1-disasm routine  2-<ea>reg 3-get <ea> routine
                   4-Dreg        5-cycle count   6-write<ea> routine                
*/


ULO i07dis(ULO prc,ULO opc,char *st) 
{
  ULO pos=13,reg=opc&SREG,dreg=(opc&DREG)>>9,o=opc&0x100,mode=(opc&SMOD)>>3,
      size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   AND.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = (o == 0) ? disfordel(reg,prc+2,st,size,mode,&pos) :
                   disfordel(dreg,prc+2,st,size,0,&pos);
  strcat(st,",");
  prc = (o != 0) ? disfordel(reg,prc,st,size,mode,&pos) :
                   disfordel(dreg,prc,st,size,0,&pos);
  return prc;
}

void i07ini(void)
{
  ULO op=0xc000,ind,regd,modes,regs,flats,size,o;
  for (o = 0; o < 2; o++)
    for (size = 0; size < 3; size++)
      for (regd = 0; regd < 8; regd++) 
        for (modes = 0; modes < 8; modes++) 
          for (regs = 0; regs < 8; regs++) {
            flats = (modes != 7) ?
                       modes:
                       (modes+regs);
            if ((o==0 && data[flats]) ||
                (o==1 && alterable[flats] && memory[flats])) {
              ind = op|regd<<9|o<<8|size<<6|modes<<3|regs;
              t[ind][0] = (o == 0) ?
                             (size == 0) ?
                                (ULO) i07_B_1:
                                (size == 1) ?
                                   (ULO) i07_W_1:
                                   (ULO) i07_L_1:
                             (size == 0) ?
                                (ULO) i07_B_2:
                                (size == 1) ?
                                   (ULO) i07_W_2:
                                   (ULO) i07_L_2;
              t[ind][1] = (ULO) i07dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO)((o == 0) ?
                             (size == 0) ?
                                arb[flats]:
                                (size == 1) ?
                                   arw[flats]:
                                   arl[flats]:
                             (size == 0) ?
                                parb[flats]:
                                (size == 1) ?
                                   parw[flats]:
                                   parl[flats]);
              t[ind][4] = regd*4 + (ULO) d;
              t[ind][5] = (o == 0) ?
                             (size == 0) ?
                                (4+tarb[flats]):
                                (size == 1) ?
                                   (4+tarw[flats]):
                                   (flats <= 1 || flats == 11) ?
                                      (8+tarl[flats]):
                                      (6+tarl[flats]):
                             (size == 0) ?
                                (8 + tarb[flats]):
                                (size == 1) ?
                                   (8 + tarw[flats]):
                                   (12 + tarl[flats]);
              if (o == 1) t[ind][6] = (ULO)((size == 0) ?
                                         awb[flats]:
                                         (size == 1) ?
                                            aww[flats]:
                                            awl[flats]);
              }
            }
}



/*
 Instruction ANDI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine               
*/


ULO i08dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  if (mode == 11 && size == 16)
    sprintf(st,"$%.6X %.4X            (PRIV) ANDI.   ",prc,opc);
  else
    sprintf(st,"$%.6X %.4X                   ANDI.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  if (mode == 11) strcat(st,(size == 8) ? "CCR":"SR");
  else prc = disfordel(reg,prc,st,size,mode,&pos);
  return prc;
}

void i08ini(void)
{
  ULO op=0x0200,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : modes+regs;
        if ((data[flats] && alterable[flats]) || (flats == 11 && size < 2)) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         ((flats == 11) ?
                           (ULO) i08_B_CCR:
                           (ULO) i08_B):
                         (size == 1) ?
                            ((flats == 11) ?
                              (ULO) i08_W_SR:
                              (ULO) i08_W):
                            (ULO) i08_L; 
          t[ind][1] = (ULO) i08dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO)((size == 0) ?
                         parb[flats]:
                         (size == 1) ?
                            parw[flats]:
                            parl[flats]);
          t[ind][4] = (flats == 11) ?
                        20:
                        (size == 0) ?
                         (flats == 0) ?
                            8:
                            (12 + tarb[flats]):
                         (size == 1)  ?
                            (flats == 0) ?
                               8:
                               (12 + tarw[flats]):
                            (flats == 0) ?
                               16:
                               (20 + tarl[flats]);
          t[ind][5] = (ULO)((size == 0) ?
                         awb[flats]:
                         (size == 1) ? 
                            aww[flats]:
                            awl[flats]);
          }
        }
}


/*
 Instruction ASL/ASR.X #i,<ea>
 3 versions: 
 Table-usage:

 ASX.X Dx,Dy
  0-Routinepointer 1-disasm routine  2-shcountreg 3-dreg
                4-cycle count               

 ASX.X #,Dy
  0-Routinepointer 1-disasm routine  2-shiftcount 3-dreg
                4-cycle count               

 ASX.X #1,<ea>
  0-Routinepointer 1-disasm routine  2-eareg 3-earead
                4-eawrite        5-cycle count               
*/


ULO i09dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=((opc&SIZE) == 0xc0) ?
                                                      64:size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  if (size == 64) {
    sprintf(st,"$%.6X %.4X                   AS .W   #$1,",prc,opc);
    prc = disfordel(reg,prc+2,st,16,mode,&pos);
    }
  else {
    if ((opc&BIT5) == 0)
      sprintf(st,"$%.6X %.4X                   AS .    #$%1X,D%1d",prc,opc,t[opc][2],reg);
    else  
      sprintf(st,"$%.6X %.4X                   AS .    D%1d,D%1d",prc,opc,(t[opc][2]-(ULO)d)>>2,(t[opc][3]-(ULO)d)>>2);
    st[35] = (size == 8) ?
                'B':
                (size == 16) ?
                   'W':
                   'L';
    prc += 2;
    }
  st[33] = ((opc&BIT8) == 0) ? 'R':'L';
  
  return prc;
}

void i09ini(void)
{
  ULO op=0xe000,ind,dir,modes,regs,regc,flats,size,immreg;
  for (size = 0; size < 3; size++)
    for (regc = 0; regc < 8; regc++) 
      for (dir = 0; dir < 2; dir++) 
        for (regs = 0; regs < 8; regs++)
          for (immreg = 0; immreg < 2; immreg++) {
            ind = op|regc<<9|dir<<8|size<<6|immreg<<5|regs;
            t[ind][0] = (size == 0) ?
                           ((immreg==0) ?
                             ((dir==0) ?
                               (ULO) i09_RI_I_B:
                               (ULO) i09_LE_I_B):
                             ((dir==0) ?
                               (ULO) i09_RI_R_B:
                               (ULO) i09_LE_R_B)):
                           ((size == 1) ?
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i09_RI_I_W:
                                 (ULO) i09_LE_I_W):
                               ((dir==0) ?
                                 (ULO) i09_RI_R_W:
                                 (ULO) i09_LE_R_W)):
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i09_RI_I_L:
                                 (ULO) i09_LE_I_L):
                               ((dir==0) ?
                                 (ULO) i09_RI_R_L:
                                 (ULO) i09_LE_R_L)));
            t[ind][1] = (ULO) i09dis;
            t[ind][2] = (immreg == 0) ?
                          ((regc == 0) ?
                            8:
                            regc):
                          (regc*4 + (ULO) d);
            t[ind][3] = regs*4 + (ULO) d;
            t[ind][4] = (size == 3) ?
                          ((immreg == 0) ?
                            (8+(t[ind][2]*2)):
                            8):
                          ((immreg == 0) ?
                            (6+(t[ind][2]*2)):
                            6);
            }


  op = 0xe0c0;
  for (dir = 0; dir < 2; dir++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : modes+regs;
        if (memory[flats] && alterable[flats]) {
          ind = op|dir<<8|modes<<3|regs;
        
          t[ind][0] = (dir == 0) ?
                        (ULO) i09_RI_M_W:
                        (ULO) i09_LE_M_W;
          t[ind][1] = (ULO) i09dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) parw[flats];
          t[ind][4] = (ULO) aww[flats];
          t[ind][5] = 8+tarw[flats];
          }
        }
}





/*
 Instruction Bcc
 Table-usage:

  0-Routinepointer 1-disasm routine  2-displacement 32 bit 3-handle routine
                4-cycle count  not taken  5-cycle count taken
*/

ULO i11dis(ULO prc,ULO opc,char *st)
{

  ULO pos=13,j,adr;
  sprintf(st,"$%.6X %.4X                   B",prc,opc);
  switch ((opc&0xf00)>>8) {
     case 0:  strcat(st,"RA.");
              break;
     case 1:  strcat(st,"SR.");
              break;
     case 2:  strcat(st,"HI.");
              break;
     case 3:  strcat(st,"LS.");
              break;
     case 4:  strcat(st,"CC.");
              break;
     case 5:  strcat(st,"CS.");
              break;
     case 6:  strcat(st,"NE.");
              break;
     case 7:  strcat(st,"EQ.");
              break;
     case 8:  strcat(st,"VC.");
              break;
     case 9:  strcat(st,"VS.");
              break;
     case 10: strcat(st,"PL.");
              break;
     case 11: strcat(st,"MI.");
              break;
     case 12: strcat(st,"GE.");
              break;
     case 13: strcat(st,"LT.");
              break;
     case 14: strcat(st,"GT.");
              break;
     case 15: strcat(st,"LE.");
              break;
       }
  strcat(st,(t[opc][2] == 0) ? "W  ":"B  ");
  if (t[opc][2] == 0) {
    prc += 2;
    j = fetw(prc);
    sprintf(&st[13],"%4.4X",j);
    st[17] = ' ';
    adr = (j > 32767) ?
             prc+j-65536:
             prc+j; 
    }
  else adr = prc+2+t[opc][2];
  sprintf(&st[36],"   $%6.6X",adr);
  
  return prc+2;
}


void i11ini(void)
{
  ULO op=0x6000,c,ind;
  int di;
  ULO routines[16] = {(ULO) cc0,(ULO) cc1,(ULO) cc2,(ULO) cc3,(ULO) cc4,(ULO) cc5,(ULO) cc6,(ULO) cc7,(ULO) cc8,(ULO) cc9,(ULO) cca,(ULO) ccb,(ULO) ccc,(ULO) ccd,(ULO) cce,(ULO) ccf};
  for (c = 0; c < 16; c++)
    for (di = -128; di < 128; di++) {
      ind = op|di&0xff|c<<8;
      t[ind][0] = (di==0) ?
                     (ULO) i11_W:
                     (ULO) i11_B;
      t[ind][1] = (ULO) i11dis;
      t[ind][2] = di;
      t[ind][3] = routines[c];
      t[ind][4] = (di==0) ? 12:8;
      t[ind][5] = 10;
      }
}

/*
 Instruction BCHG.B/L 
 Table-usage dynamic:
 2 - eareg 3-earead 4-reg 5-cycle count 6-eawrite
 Table-usage static:
 2 - eareg 3-earead 4-cycle count 5-eawrite
*/

ULO i12dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=18-(5*((opc&BIT8)>>8)),reg=opc&SREG,bitreg=(opc&DREG)>>9,
      mode=(opc&SMOD)>>3;

  if (mode == 7) mode += reg;
  if (pos == 18) {
    sprintf(st,"$%.6X %.4X %.4X              BCHG.%s  #$%.4X,",prc,opc,fetw(prc+2),
            (mode == 0) ? "L":"B",fetw(prc+2)&((mode == 0) ? 0x1f:0x7));
    prc += 2;
    }
  else sprintf(st,"$%.6X %.4X                   BCHG.%s  D%1X,",prc,opc,
               (mode == 0) ? "L":"B",bitreg);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}


void i12ini(void)
{
  ULO op=0x0140,ind,breg,mode,reg,flat;

/* Dynamic first */

  for (breg = 0; breg < 8; breg++)
    for (mode = 0; mode < 8; mode++) 
      for (reg = 0; reg < 8; reg++) {
        flat = (mode != 7) ? mode : mode+reg;
        if (data[flat] && alterable[flat]) {
          ind = op|breg<<9|mode<<3|reg;
          t[ind][0] = (mode == 0) ? (ULO) i12_D_L:
                                    (ULO) i12_D_B;
          t[ind][1] = (ULO) i12dis;
          t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                             (ULO) a);
          t[ind][3] = (ULO) parb[flat];
          t[ind][4] = breg*4 + (ULO) d;
          t[ind][5] = (mode == 0) ? 8:
                                    (8+tarb[flat]);
          t[ind][6] = (ULO) awb[flat];
          }
        }
/* Static */
  op=0x0840;
  for (mode = 0; mode < 8; mode++)
    for (reg = 0; reg < 8; reg++) {
      flat = (mode != 7) ? mode : mode+reg;
      if (data[flat] && alterable[flat]) {
        ind = op|mode<<3|reg;
        t[ind][0] = (mode == 0) ? (ULO) i12_S_L:
                                  (ULO) i12_S_B;
        t[ind][1] = (ULO) i12dis;
        t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                           (ULO) a);
        t[ind][3] = (ULO) parb[flat];
        t[ind][4] = (mode == 0) ? 12:
                                  (12+tarb[flat]);
        t[ind][5] = (ULO) awb[flat];
        }
      }
}

/*
 Instruction BCLR.B/L 
 Table-usage dynamic:
 2 - eareg 3-earead 4-reg 5-cycle count 6-eawrite
 Table-usage static:
 2 - eareg 3-earead 4-cycle count 5-eawrite
*/

ULO i13dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=18-(5*((opc&BIT8)>>8)),reg=opc & SREG,bitreg=(opc & DREG)>>9,mode=(opc & SMOD)>>3;

  if (mode == 7) mode += reg;
  if (pos == 18) {
    sprintf(st,"$%.6X %.4X %.4X              BCLR.%s  #$%.4X,",prc,opc,fetw(prc+2),
            (mode == 0) ? "L":"B",fetw(prc+2)&((mode == 0) ? 0x1f:0x7));
    prc += 2;
    }
  else sprintf(st,"$%.6X %.4X                   BCLR.%s  D%1X,",prc,opc,
               (mode == 0) ? "L":"B",bitreg);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}


void i13ini(void)
{
  ULO op=0x0180,ind,breg,mode,reg,flat;

/* Dymnamisk frst */

  for (breg = 0; breg < 8; breg++)
    for (mode = 0; mode < 8; mode++) 
      for (reg = 0; reg < 8; reg++) {
        flat = (mode != 7) ? mode : mode+reg;
        if (data[flat] && alterable[flat]) {
          ind = op|breg<<9|mode<<3|reg;
          t[ind][0] = (mode == 0) ? (ULO) i13_D_L:
                                    (ULO) i13_D_B;
          t[ind][1] = (ULO) i13dis;
          t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                             (ULO) a);
          t[ind][3] = (ULO) ((mode == 0) ? parl[flat]:
                                    parb[flat]);
          t[ind][4] = breg*4 + (ULO) d;
          t[ind][5] = (mode == 0) ? 10:
                                    (8+tarb[flat]);
          t[ind][6] = (ULO)((mode == 0) ? awl[flat]:
                                    awb[flat]);
          }
        }
/* Static */
  op=0x0880;
  for (mode = 0; mode < 8; mode++)
    for (reg = 0; reg < 8; reg++) {
      flat = (mode != 7) ? mode : mode+reg;
      if (data[flat] && alterable[flat]) {
        ind = op|mode<<3|reg;
        t[ind][0] = (mode == 0) ? (ULO) i13_S_L:
                                  (ULO) i13_S_B;
        t[ind][1] = (ULO) i13dis;
        t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                           (ULO) a);
        t[ind][3] = (ULO)((mode == 0) ? parl[flat]:
                                  parb[flat]);
        t[ind][4] = (mode == 0) ? 14:
                                  (12+tarb[flat]);
        t[ind][5] = (ULO)((mode == 0) ? awl[flat]:
                                  awb[flat]);
        }
      }
}

/*
 Instruction BSET.B/L 
 Table-usage dynamic:
 2 - eareg 3-earead 4-reg 5-cycle count 6-eawrite
 Table-usage static:
 2 - eareg 3-earead 4-cycle count 5-eawrite
*/

ULO i15dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=18-(5*((opc&BIT8)>>8)),reg=opc & SREG,bitreg=(opc & DREG)>>9,mode=(opc & SMOD)>>3;

  if (mode == 7) mode += reg;
  if (pos == 18) {
    sprintf(st,"$%.6X %.4X %.4X              BSET.%s  #$%.4X,",prc,opc,fetw(prc+2),
            (mode == 0) ? "L":"B",fetw(prc+2)&((mode == 0) ? 0x1f:0x7));
    prc += 2;
    }
  else sprintf(st,"$%.6X %.4X                   BSET.%s  D%1X,",prc,opc,
               (mode == 0) ? "L":"B",bitreg);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}


void i15ini(void)
{
  ULO op=0x01c0,ind,breg,mode,reg,flat;

/* Dynamic first */

  for (breg = 0; breg < 8; breg++)
    for (mode = 0; mode < 8; mode++) 
      for (reg = 0; reg < 8; reg++) {
        flat = (mode != 7) ? mode : mode+reg;
        if (data[flat] && alterable[flat]) {
          ind = op|breg<<9|mode<<3|reg;
          t[ind][0] = (mode == 0) ? (ULO) i15_D_L:
                                    (ULO) i15_D_B;
          t[ind][1] = (ULO) i15dis;
          t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                             (ULO) a);
          t[ind][3] = (ULO)((mode == 0) ? parl[flat]:
                                    parb[flat]);
          t[ind][4] = breg*4 + (ULO) d;
          t[ind][5] = (mode == 0) ? 8:
                                    (8+tarb[flat]);
          t[ind][6] = (ULO)((mode == 0) ? awl[flat]:
                                    awb[flat]);
          }
        }
/* Static */
  op=0x08c0;
  for (mode = 0; mode < 8; mode++)
    for (reg = 0; reg < 8; reg++) {
      flat = (mode != 7) ? mode : mode+reg;
      if (data[flat] && alterable[flat]) {
        ind = op|mode<<3|reg;
        t[ind][0] = (mode == 0) ? (ULO) i15_S_L:
                                  (ULO) i15_S_B;
        t[ind][1] = (ULO) i15dis;
        t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                           (ULO) a);
        t[ind][3] = (ULO)((mode == 0) ? parl[flat]:
                                  parb[flat]);
        t[ind][4] = (mode == 0) ? 12:
                                  (12+tarb[flat]);
        t[ind][5] = (ULO)((mode == 0) ? awl[flat]:
                                  awb[flat]);
        }
      }
}

/*
 Instruction BTST.B/L 
 Table-usage dynamic:
 2 - eareg 3-earead 4-reg 5-cycle count
 Table-usage static:
 2 - eareg 3-earead 4-cycle count
*/

ULO i17dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=18-(5*((opc&BIT8)>>8)),reg=opc & SREG,bitreg=(opc & DREG)>>9,mode=(opc & SMOD)>>3;

  if (mode == 7) mode += reg;
  if (pos == 18) {
    sprintf(st,"$%.6X %.4X %.4X              BTST.%s  #$%.4X,",prc,opc,fetw(prc+2),
            (mode == 0) ? "L":"B",fetw(prc+2)&((mode == 0) ? 0x1f:0x7));
    prc += 2;
    }
  else sprintf(st,"$%.6X %.4X                   BTST.%s  D%1X,",prc,opc,
               (mode == 0) ? "L":"B",bitreg);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}


void i17ini(void)
{
  ULO op=0x0100,ind,breg,mode,reg,flat;

/* Dynamic first */

  for (breg = 0; breg < 8; breg++)
    for (mode = 0; mode < 8; mode++) 
      for (reg = 0; reg < 8; reg++) {
        flat = (mode != 7) ? mode : mode+reg;
        if (data[flat]) {
          ind = op|breg<<9|mode<<3|reg;
          t[ind][0] = (mode == 0) ? (ULO) i17_D_L:
                                    (ULO) i17_D_B;
          t[ind][1] = (ULO) i17dis;
          t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                             (ULO) a);
          t[ind][3] = (ULO)((mode == 0) ? arl[flat]:
                                    arb[flat]);
          t[ind][4] = breg*4 + (ULO) d;
          t[ind][5] = (mode == 0) ? 6:
                                    (4+tarb[flat]);
          }
        }
/* Static */
  op=0x0800;
  for (mode = 0; mode < 8; mode++)
    for (reg = 0; reg < 8; reg++) {
      flat = (mode != 7) ? mode : mode+reg;
      if (data[flat] && (flat != 12)) {
        ind = op|mode<<3|reg;
        t[ind][0] = (mode == 0) ? (ULO) i17_S_L:
                                  (ULO) i17_S_B;
        t[ind][1] = (ULO) i17dis;
        t[ind][2] = reg*4 + ((mode == 0) ? (ULO) d:
                                           (ULO) a);
        t[ind][3] = (ULO)((mode == 0) ? arl[flat]:
                                  arb[flat]);
        t[ind][4] = (mode == 0) ? 10:
                                  (8+tarb[flat]);
        }
      }
}

/*
 Instruction CHK.W <ea>,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5 - cycle count                     
*/

ULO i18dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   CHK.W   ",prc,opc);
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,16,0,&pos);
  
  return prc;
}

void i18ini(void)
{
  ULO op=0x4180,ind,regd,modes,regs,flats;
  for (regd = 0; regd < 8; regd++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) 
        if (data[(flats = (modes != 7) ? modes : modes+regs)]) {
          ind = op|regd<<9|modes<<3|regs;
          t[ind][0] = (ULO) i18;
          t[ind][1] = (ULO) i18dis;
          t[ind][2] = regs*4 + ((modes == 0) ?
                                  (ULO) d:
                                  (ULO) a);
          t[ind][3] = (ULO) arw[flats];
          t[ind][4] = regd*4 + (ULO) d;
          t[ind][5] = 10+tarw[flats];
          }
}

/*
 Instruction CLR.X <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-cycle count                   
*/

ULO i19dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   CLR.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}

void i19ini(void)
{
  ULO op=0x4200,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   modes+regs;
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
        t[ind][0] = (ULO) i19;
        t[ind][1] = (ULO) i19dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO)((size == 0) ?
                       awb[flats]:
                       (size == 1) ?
                          aww[flats]:
                          awl[flats]);
        t[ind][4] = (size == 0) ?
                       4+tarb[flats]:
                       (size == 1) ?
                          (4+tarw[flats]):
                          (modes == 0) ?
                             6:
                             (4+tarl[flats]);
        }
      }
}

/*
 Instruction CMP.X <ea>,Dn / CMP.X Dn,<ea>
 Table-usage:
 Type 1:  CMP <ea>,Dn
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5-cycle count                   
*/

ULO i20dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3,
      size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   CMP.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,size,0,&pos);
  
  return prc;
}

void i20ini(void)
{
  ULO op=0xb000,ind,regd,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) {
          flats = (modes != 7) ?
                     modes:
                     (modes+regs);
          if ((size == 0 && data[flats]) ||
              (size != 0 && allmodes[flats])) {
            ind = op|regd<<9|size<<6|modes<<3|regs;
            t[ind][0] = (size == 0) ?
                           (ULO) i20_B:
                           (size == 1) ?
                              (ULO) i20_W:
                              (ULO) i20_L;
            t[ind][1] = (ULO) i20dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO)((size == 0) ?
                           arb[flats]:
                           (size == 1) ?
                              arw[flats]:
                              arl[flats]);
            t[ind][4] = regd*4 + (ULO) d;
            t[ind][5] = (size == 0) ?
                           (4+tarb[flats]):
                           (size == 1) ?
                              (4+tarw[flats]):
                              (6+tarl[flats]);
            }
          }
}

/*
 Instruction CMPA.W/L <ea>,An
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Areg        5-cycle count                   
*/

ULO i21dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,o=opc&0x100,mode=(opc&0x38)>>3,size;
  size = (o == 0) ?
            16:
            32;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   CMPA.   ",prc,opc);
  st[36] = (size == 16) ?
              'W':
              'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,size,1,&pos);
  
  return prc;
}

void i21ini(void)
{
  ULO op=0xb0c0,ind,regd,modes,regs,flats,o;
  for (o = 0; o < 2; o++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) 
          if (allmodes[(flats = (modes != 7) ? modes : (modes+regs))]) {
            ind = op|regd<<9|o<<8|modes<<3|regs;
            t[ind][0] = (o == 0) ?
                           (ULO) i21_W:
                           (ULO) i21_L;
            t[ind][1] = (ULO) i21dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO)((o == 0) ?
                           arw[flats]:
                           arl[flats]);
            t[ind][4] = regd*4 + (ULO) a;
            t[ind][5] = (o==0) ?
                           (6 + tarw[flats]):
                           (6 + tarl[flats]);
            }
}


/*
 Instruction CMPI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count               
*/

ULO i22dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   CMPI.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  prc = disfordel(reg,prc,st,size,mode,&pos);
  
  return prc;
}

void i22ini(void)
{
  ULO op=0x0c00,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         (ULO) i22_B:
                         (size == 1) ?
                            (ULO) i22_W:
                            (ULO) i22_L; 
          t[ind][1] = (ULO) i22dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO)((size == 0) ?
                         arb[flats]:
                         (size == 1) ?
                            arw[flats]:
                            arl[flats]);
          t[ind][4] = (size == 0) ?
                         (8 + tarb[flats]):
                         (size == 1)  ?
                            (8 + tarw[flats]):
                            (flats == 0) ?
                               14:
                               (12 + tarl[flats]);
          }
        }
}

/*
 Instruction CMPM.X (Ay)+,(Ax)+
 Table-usage: 0 - Routinepointer 1 - disasm routine 2 - sou 3 - dest
*/

ULO i23dis(ULO prc,ULO opc,char *st)
{

  ULO size=(opc&0xc0)>>6;
  sprintf(st,("$%.6X %.4X                   CMPX.%s  (A%1d)+,(A%1d)+"),
             prc,opc,(size == 0) ? "B":((size == 1) ? "W":"L"),opc&SREG,(opc&DREG)>>9);
  
  return prc+2;
}

void i23ini(void)
{
  ULO op=0xb108,ind,rx,ry,sz;
  for (sz = 0; sz <= 2; sz++)
      for (rx = 0; rx <= 7; rx++) 
        for (ry = 0; ry <= 7; ry++) {
          ind = op|ry|rx<<9|sz<<6;
        t[ind][0] = (sz == 0) ?
                      (ULO) i23_B:
                      (sz == 1) ?
                        (ULO) i23_W:
                        (ULO) i23_L;
        t[ind][1] = (ULO) i23dis;
        t[ind][2] = (ry*4 + (ULO) a);
        t[ind][3] = (rx*4 + (ULO) a);
        }
}
/*
 Instruction DBcc Dn,offset
 Table-usage:

  0-Routinepointer 1-disasm routine  2-displacement 32 bit 3-handle routine
*/

ULO i24dis(ULO prc,ULO opc,char *st)
{

  ULO pos=13,j,adr;
  prc += 2;
  j = fetw(prc);
  adr = (j > 32767) ?
           prc+j-65536:
           prc+j; 
  sprintf(st,"$%.6X %.4X %.4X              DB",prc-2,opc,j);
  switch ((opc&0xf00)>>8) {
     case 0:  strcat(st,"T    ");
              break;
     case 1:  strcat(st,"F    ");
              break;
     case 2:  strcat(st,"HI   ");
              break;
     case 3:  strcat(st,"LS   ");
              break;
     case 4:  strcat(st,"CC   ");
              break;
     case 5:  strcat(st,"CS   ");
              break;
     case 6:  strcat(st,"NE   ");
              break;
     case 7:  strcat(st,"EQ   ");
              break;
     case 8:  strcat(st,"VC   ");
              break;
     case 9:  strcat(st,"VS   ");
              break;
     case 10: strcat(st,"PL   ");
              break;
     case 11: strcat(st,"MI   ");
              break;
     case 12: strcat(st,"GE   ");
              break;
     case 13: strcat(st,"LT   ");
              break;
     case 14: strcat(st,"GT   ");
              break;
     case 15: strcat(st,"LE   ");
              break;
       }
  sprintf(&st[36],"   D%1d,$%6.6X",opc&0x7,adr);
  
  return prc+2;
}


void i24ini(void)
{
  ULO op=0x50c8,c,ind,reg;
  ULO routines[16] = {(ULO) cc0,(ULO) cc1false,(ULO) cc2,(ULO) cc3,(ULO) cc4,(ULO) cc5,(ULO) cc6,(ULO) cc7,(ULO) cc8,(ULO) cc9,(ULO) cca,(ULO) ccb,(ULO) ccc,(ULO) ccd,(ULO) cce,(ULO) ccf};
  for (c = 0; c < 16; c++)
    for (reg = 0; reg < 8; reg++) {
      ind = op|reg|c<<8;
      t[ind][0] = (ULO) i24;
      t[ind][1] = (ULO) i24dis;
      t[ind][2] = (reg*4) + (ULO) d;
      t[ind][3] = routines[c];
      }
}

/*
 Instruction DIVS.W <ea>,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg                          
*/

ULO i25dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   DIVS.W  ",prc,opc);
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,16,0,&pos);
  
  return prc;
}

void i25ini(void)
{
  ULO op=0x81c0,ind,regd,modes,regs,flats;
  for (regd = 0; regd < 8; regd++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) 
        if (data[(flats = (modes != 7) ? modes : (modes+regs))]) {
          ind = op|regd<<9|modes<<3|regs;
          t[ind][0] = (ULO) i25;
          t[ind][1] = (ULO) i25dis;
          t[ind][2] = regs*4 + ((modes == 0) ?
                                  (ULO) d:
                                  (ULO) a);
          t[ind][3] = (ULO) arw[flats];
          t[ind][4] = regd*4 + (ULO) d;
          }
}

/*
 Instruction DIVU.W <ea>,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg                          
*/

ULO i26dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   DIVU.W  ",prc,opc);
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,16,0,&pos);
  
  return prc;
}

void i26ini(void)
{
  ULO op=0x80c0,ind,regd,modes,regs,flats;
  for (regd = 0; regd < 8; regd++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) 
        if (data[(flats = (modes != 7) ? modes : (modes+regs))]) {
          ind = op|regd<<9|modes<<3|regs;
          t[ind][0] = (ULO) i26;
          t[ind][1] = (ULO) i26dis;
          t[ind][2] = regs*4 + ((modes == 0) ?
                                  (ULO) d:
                                  (ULO) a);
          t[ind][3] = (ULO) arw[flats];
          t[ind][4] = regd*4 + (ULO) d;
          }
}

/*
 Instruction EOR.X <ea>,Dn / EOR.X Dn,<ea>
 Table-usage:
 EOR Dn,<ea>
     0-Routinepointer 1-disasm routine  2-<ea>reg 3-get <ea> routine
                   4-Dreg        5-cycle count   6-write<ea> routine                
*/

ULO i27dis(ULO prc,ULO opc,char *st)
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,o=opc&0x100,mode=(opc&0x38)>>3,
      size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   EOR.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel((opc&0xe00)>>9,prc+2,st,size,0,&pos);
  strcat(st,",");
  prc = disfordel(reg,prc,st,size,mode,&pos);
  
  return prc;
}

void i27ini(void)
{
  ULO op=0xb100,ind,regd,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) {
          flats = (modes != 7) ?
                     modes:
                     (modes+regs);
          if (data[flats] && alterable[flats]) {
            ind = op|regd<<9|size<<6|modes<<3|regs;
            t[ind][0] = (size == 0) ?
                           (ULO) i27_B:
                           (size == 1) ?
                              (ULO) i27_W:
                              (ULO) i27_L;
            t[ind][1] = (ULO) i27dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO)((size == 0) ?
                           parb[flats]:
                           (size == 1) ?
                              parw[flats]:
                              parl[flats]);
            t[ind][4] = regd*4 + (ULO) d;
            t[ind][5] = (size == 0) ?
                           (8 + tarb[flats]):
                           (size == 1) ?
                              (8 + tarw[flats]):
                              (12 + tarl[flats]);
            t[ind][6] = (ULO)((size == 0) ?
                           awb[flats]:
                           (size == 1) ?
                              aww[flats]:
                              awl[flats]);
            }
          }
}



/*
 Instruction EORI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine               
*/


ULO i28dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  if (mode == 11 && size == 16)
    sprintf(st,"$%.6X %.4X            (PRIV) EORI.   ",prc,opc);
  else
    sprintf(st,"$%.6X %.4X                   EORI.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  if (mode == 11) strcat(st,(size == 8) ? "CCR":"SR");
  else prc = disfordel(reg,prc,st,size,mode,&pos);
  
  return prc;
}

void i28ini(void)
{
  ULO op=0x0a00,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if ((data[flats] && alterable[flats]) || (flats == 11 && size < 2)) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         ((flats == 11) ?
                           (ULO) i28_B_CCR:
                           (ULO) i28_B):
                         (size == 1) ?
                            ((flats == 11) ?
                              (ULO) i28_W_SR:
                              (ULO) i28_W):
                            (ULO) i28_L; 
          t[ind][1] = (ULO) i28dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO)((size == 0) ?
                         parb[flats]:
                         (size == 1) ?
                            parw[flats]:
                            parl[flats]);
          t[ind][4] = (flats == 11) ?
                        20:
                        (size == 0) ?
                         (flats == 0) ?
                            8:
                            (12 + tarb[flats]):
                         (size == 1)  ?
                            (flats == 0) ?
                               8:
                               (12 + tarw[flats]):
                            (flats == 0) ?
                               16:
                               (20 + tarl[flats]);
          t[ind][5] = (ULO)((size == 0) ?
                         awb[flats]:
                         (size == 1) ? 
                            aww[flats]:
                            awl[flats]);
          }
        }
}


/*
 Instruction exg Rx,Rx
 Table-usage:

  0-Routinepointer 1-disasm routine  2-reg1 3-reg2                   

*/

ULO i29dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,o=(opc&0x00f8)>>3,reg1=(opc&0xe00)>>9,reg2=opc&7;
  if (o == 8) 
    sprintf(st,"$%.6X %.4X                   EXG.L   D%d,D%d",prc,opc,reg1,reg2);
  else if (o == 9)
    sprintf(st,"$%.6X %.4X                   EXG.L   A%d,A%d",prc,opc,reg1,reg2);
  else
    sprintf(st,"$%.6X %.4X                   EXG.L   A%d,D%d",prc,opc,reg1,reg2);
  
  return prc+2;
}

void i29ini(void)
{
  ULO op=0xc100,ind,reg1,reg2;

  for (reg2 = 0; reg2 < 8; reg2++)
    for (reg1 = 0; reg1 < 8; reg1++) {
      ind = op|0x8<<3|reg2<<9|reg1;
      t[ind][0] = (ULO) i29;
      t[ind][1] = (ULO) i29dis;
      t[ind][2] = reg1*4 + (ULO) d;
      t[ind][3] = reg2*4 + (ULO) d;
      }

  for (reg2 = 0; reg2 < 8; reg2++)
    for (reg1 = 0; reg1 < 8; reg1++) {
      ind = op|0x9<<3|reg2<<9|reg1;
      t[ind][0] = (ULO) i29;
      t[ind][1] = (ULO) i29dis;
      t[ind][2] = reg1*4 + (ULO) a;
      t[ind][3] = reg2*4 + (ULO) a;
      }

  for (reg2 = 0; reg2 < 8; reg2++)
    for (reg1 = 0; reg1 < 8; reg1++) {
      ind = op|0x11<<3|reg2<<9|reg1;
      t[ind][0] = (ULO) i29;
      t[ind][1] = (ULO) i29dis;
      t[ind][2] = reg1*4 + (ULO) a;
      t[ind][3] = reg2*4 + (ULO) d;
      }
}

/*
 Instruction ext Dx
 Table-usage:

  0-Routinepointer 1-disasm routine  2-reg 3-cycle count                   

*/

ULO i30dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,o=opc&BIT6,reg=opc&SREG;
  sprintf(st,"$%.6X %.4X                   EXT.%s   D%d",prc,opc,(o==0) ? "W":"L",reg);
  
  return prc+2;
}

void i30ini(void)
{
  ULO op=0x4880,ind,o,regs;
  for (o = 0; o < 2; o++) 
    for (regs = 0; regs < 8; regs++) {
      ind = op|o<<6|regs;
      t[ind][0] = (o == 0) ? (ULO) i30_W:(ULO) i30_L;
      t[ind][1] = (ULO) i30dis;
      t[ind][2] = regs*4 + (ULO) d;
      }
}

/*
 Instruction jmp <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceread
                   4-cycle count                   
*/

ULO i31dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   JMP     ",prc,opc);
  prc = disfordel(reg,prc+2,st,32,mode,&pos);
  
  return prc;
}

void i31ini(void)
{
  ULO op=0x4ec0,ind,modes,regs,flats;
  for (modes = 0; modes < 8; modes++) 
    for (regs = 0; regs < 8; regs++) {
      flats = (modes != 7) ? modes : (modes+regs);
      if (control[flats]) {
        ind = op|modes<<3|regs;
        t[ind][0] = (ULO) i31;
        t[ind][1] = (ULO) i31dis;
        t[ind][2] = regs*4 + (ULO) a;
        t[ind][3] = (ULO) eac[flats];
        t[ind][4] = (flats == 2) ? 8:
                      (flats == 5) ? 10:
                        (flats == 6) ? 14:
                          (flats == 7) ? 10:
                            (flats == 9) ? 12:
                              (flats == 10) ? 10:14;
        }
      }
}

/*
 Instruction jsr <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceread
                   4-cycle count                   
*/

ULO i32dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   JSR     ",prc,opc);
  prc = disfordel(reg,prc+2,st,32,mode,&pos);
  
  return prc;
}

void i32ini(void)
{
  ULO op=0x4e80,ind,modes,regs,flats;
  for (modes = 0; modes < 8; modes++) 
    for (regs = 0; regs < 8; regs++) {
      flats = (modes != 7) ? modes : (modes+regs);
      if (control[flats]) {
        ind = op|modes<<3|regs;
        t[ind][0] = (ULO) i32;
        t[ind][1] = (ULO) i32dis;
        t[ind][2] = regs*4 + (ULO) a;
        t[ind][3] = (ULO) eac[flats];
        t[ind][4] = (flats == 2) ? 8:
                      (flats == 5) ? 10:
                        (flats == 6) ? 14:
                          (flats == 7) ? 10:
                            (flats == 9) ? 12:
                              (flats == 10) ? 10:14;
        }
      }
}

/*
 Instruction lea <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-earead
                   4-Areg 5-cycle count                   
*/

ULO i33dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,areg=(opc&DREG)>>9,reg=opc&SREG,mode=(opc&SMOD)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   LEA.L   ",prc,opc);
  prc = disfordel(reg,prc+2,st,32,mode,&pos);
  strcat(st,",");
  prc = disfordel(areg,prc,st,32,1,&pos);
  
  return prc;
}

void i33ini(void)
{
  ULO op=0x41c0,ind,areg,modes,regs,flats;
  for (areg = 0; areg < 8; areg++)
  for (modes = 0; modes < 8; modes++)
    for (regs = 0; regs < 8; regs++) {
      flats = (modes != 7) ? modes : (modes+regs);
      if (control[flats]) {
        ind = op|areg<<9|modes<<3|regs;
        t[ind][0] = (ULO) i33;
        t[ind][1] = (ULO) i33dis;
        t[ind][2] = regs*4 + (ULO) a;
        t[ind][3] = (ULO) eac[flats];
        t[ind][4] = areg*4 + (ULO) a;
        t[ind][5] = (flats == 2) ? 4:
                      (flats == 5) ? 8:
                        (flats == 6) ? 12:
                          (flats == 7) ? 8:
                            (flats == 8) ? 12:
                              (flats == 9) ? 8:12;
        }
      }
}


/*
 Instruction LINK An,#
 Table-usage:
     0-Routinepointer 1-disasm routine  2-reg
*/


ULO i34dis(ULO prc,ULO opc,char *st) 
{

  ULO j,pos=13,reg=opc&0x7;
  j=fetw(prc+2);
  sprintf(st,"$%.6X %.4X %.4X              LINK    A%1d,#$%.4X",prc,opc,j,reg,j);
  
  return prc+4;
}

void i34ini(void)
{
  ULO op=0x4e50,ind,reg;
  for (reg = 0; reg < 8; reg++) {
    ind = op|reg;
    t[ind][0] = (ULO) i34;
    t[ind][1] = (ULO) i34dis;
    t[ind][2] = reg*4 + (ULO) a;
    }    
}

/*
 Instruction LSL/LSR.X #i,<ea>
 3 versions: 
 Table-usage:

 LSX.X Dx,Dy
  0-Routinepointer 1-disasm routine  2-shcountreg 3-dreg
                4-cycle count               

 LSX.X #,Dy
  0-Routinepointer 1-disasm routine  2-shiftcount 3-dreg
                4-cycle count               

 LSX.X #1,<ea>
  0-Routinepointer 1-disasm routine  2-eareg 3-earead
                4-eawrite        5-cycle count               
*/


ULO i35dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=((opc&SIZE) == 0xc0) ?
                                                      64:size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  if (size == 64) {
    sprintf(st,"$%.6X %.4X                   LS .W   #$1,",prc,opc);
    prc = disfordel(reg,prc+2,st,16,mode,&pos);
    }
  else {
    if ((opc&BIT5) == 0)
      sprintf(st,"$%.6X %.4X                   LS .    #$%1X,D%1d",prc,opc,t[opc][2],(t[opc][3]-(ULO)d)>>2);
    else  
      sprintf(st,"$%.6X %.4X                   LS .    D%1d,D%1d",prc,opc,(t[opc][2]-(ULO)d)>>2,(t[opc][3]-(ULO)d)>>2);
    st[35] = (size == 8) ?
                'B':
                (size == 16) ?
                   'W':
                   'L';
    prc += 2;
    }
  st[33] = ((opc&BIT8) == 0) ? 'R':'L';
  
  return prc;
}

void i35ini(void)
{
  ULO op=0xe008,ind,dir,modes,regs,regc,flats,size,immreg;
  for (size = 0; size < 3; size++)
    for (regc = 0; regc < 8; regc++) 
      for (dir = 0; dir < 2; dir++) 
        for (regs = 0; regs < 8; regs++)
          for (immreg = 0; immreg < 2; immreg++) {
            ind = op|regc<<9|dir<<8|size<<6|immreg<<5|regs;
            t[ind][0] = (size == 0) ?
                           ((immreg==0) ?
                             ((dir==0) ?
                               (ULO) i35_RI_I_B:
                               (ULO) i35_LE_I_B):
                             ((dir==0) ?
                               (ULO) i35_RI_R_B:
                               (ULO) i35_LE_R_B)):
                           ((size == 1) ?
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i35_RI_I_W:
                                 (ULO) i35_LE_I_W):
                               ((dir==0) ?
                                 (ULO) i35_RI_R_W:
                                 (ULO) i35_LE_R_W)):
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i35_RI_I_L:
                                 (ULO) i35_LE_I_L):
                               ((dir==0) ?
                                 (ULO) i35_RI_R_L:
                                 (ULO) i35_LE_R_L)));
            t[ind][1] = (ULO) i35dis;
            t[ind][2] = (immreg == 0) ?
                          ((regc == 0) ?
                            8:
                            regc):
                          (regc*4 + (ULO) d);
            t[ind][3] = regs*4 + (ULO) d;
            t[ind][4] = (size == 3) ?
                          ((immreg == 0) ?
                            (8+(t[ind][2]*2)):
                            8):
                          ((immreg == 0) ?
                            (6+(t[ind][2]*2)):
                            6);
            }


  op = 0xe2c0;
  for (dir = 0; dir < 2; dir++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (memory[flats] && alterable[flats]) {
          ind = op|dir<<8|modes<<3|regs;
        
          t[ind][0] = (dir == 0) ?
                        (ULO) i35_RI_M_W:
                        (ULO) i35_LE_M_W;
          t[ind][1] = (ULO) i35dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) parw[flats];
          t[ind][4] = (ULO) aww[flats];
          t[ind][5] = 8+tarw[flats];
          }
        }
}





/*
 Instruction MOVE.X <ea>,<ea>
 Table-usage: 0-Routinepointer 1-disasm routine 2-sourcereg 3-sourcerut
                   4-destreg     5-destwrite 6-cycle count                   
*/

ULO i37dis(ULO prc,ULO opc,char *st) 
{

  ULO mode=(opc&SMOD)>>3,i,pos=13,reg=opc&SREG,size=opc&0x3000;
  sprintf(st,"$%.6X %.4X                   MOVE.   ",prc,opc);
  size = (size == 0x1000) ?
            8:
            (size == 0x3000) ?
               16:
               32;
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  if (mode == 7) mode += reg;
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  mode = (opc & 0x01c0)>>6;
  if (mode == 7) mode += (opc & 0x0e00)>>9;
  prc = disfordel((opc & 0x0e00)>>9,prc,st,size,mode,&pos);
  
  return prc;
}

void i37ini(void)
{
  ULO op=0x0000,ind,moded,regd,modes,regs,size,flatd,flats;
  size = 0x1000; /* Byte */
  for (moded = 0; moded < 8; moded++) {
    for (regd = 0; regd < 8; regd++) {
      if (moded != 7) flatd = moded;
      else flatd = moded+regd;
      if (data[flatd] && alterable[flatd]) {
        for (modes = 0; modes < 8; modes++) {
          for (regs = 0; regs < 8; regs++) {
            if (modes != 7) flats = modes;
            else flats = modes+regs;
            if (data[flats]) {
              ind = op|size|regd<<9|moded<<6|modes<<3|regs;
              t[ind][0] = (ULO) i37_B;
              t[ind][1] = (ULO) i37dis;
              if (modes == 0) t[ind][2] = regs*4 + (ULO) d;
              else t[ind][2] = regs*4 + (ULO) a;
              t[ind][3] = (ULO) arb[flats];
              if (moded == 0) t[ind][4] = regd*4 + (ULO) d;
              else t[ind][4] = regd*4 + (ULO) a;
              t[ind][5] = (ULO) awb[flatd];
              t[ind][6] = 4+tarb[flats]+tarb[flatd];
              }
            }
          }
        }
      }
    }
  size = 0x3000; /* Word */
  for (moded = 0; moded < 8; moded++) {
    for (regd = 0; regd < 8; regd++) {
      if (moded != 7) flatd = moded;
      else flatd = moded+regd;
      if (data[flatd] && alterable[flatd]) {
        for (modes = 0; modes < 8; modes++) {
          for (regs = 0; regs < 8; regs++) {
            if (modes != 7) flats = modes;
            else flats = modes+regs;
            if (allmodes[flats]) {
              ind = op|size|regd<<9|moded<<6|modes<<3|regs;
              t[ind][0] = (ULO) i37_W;
              t[ind][1] = (ULO) i37dis;
              if (modes == 0) t[ind][2] = regs*4 + (ULO) d;
              else t[ind][2] = regs*4 + (ULO) a;
              t[ind][3] = (ULO) arw[flats];
              if (moded == 0) t[ind][4] = regd*4 + (ULO) d;
              else t[ind][4] = regd*4 + (ULO) a;
              t[ind][5] = (ULO) aww[flatd];
              t[ind][6] = 4+tarw[flats]+tarw[flatd];
              }
            }
          }
        }
      }
    }
  size = 0x2000; /* Long */
  for (moded = 0; moded < 8; moded++) {
    for (regd = 0; regd < 8; regd++) {
      if (moded != 7) flatd = moded;
      else flatd = moded+regd;
      if (data[flatd] && alterable[flatd]) {
        for (modes = 0; modes < 8; modes++) {
          for (regs = 0; regs < 8; regs++) {
            if (modes != 7) flats = modes;
            else flats = modes+regs;
            if (allmodes[flats]) {
              ind = op|size|regd<<9|moded<<6|modes<<3|regs;
              t[ind][0] = (ULO) i37_L;
              t[ind][1] = (ULO) i37dis;
              if (modes == 0) t[ind][2] = regs*4 + (ULO) d;
              else t[ind][2] = regs*4 + (ULO) a;
              t[ind][3] = (ULO) arl[flats];
              if (moded == 0) t[ind][4] = regd*4 + (ULO) d;
              else t[ind][4] = regd*4 + (ULO) a;
              t[ind][5] = (ULO) awl[flatd];
              t[ind][6] = 4+tarl[flats]+tarl[flatd];
              }
            }
          }
        }
      }
    }
}

/*
 Instruction MOVE.B to CCR
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourcereg 3-sourceroutine
                4-cycle count                   
*/

ULO i38dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  sprintf(st,"$%.6X %.4X                   MOVE.B  ",prc,opc);
  if (mode == 7) mode += reg;
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  strcat(st,",CCR");
  
  return prc;
}

void i38ini(void)
{
  ULO op=0x44c0,ind,reg,mode,flat;
    for (reg = 0; reg < 8; reg++) 
      for (mode = 0; mode < 8; mode++) {
        flat = (mode != 7) ? mode : (mode+reg);
        if (data[flat]) {
            ind = op|mode<<3|reg;
            t[ind][0] = (ULO) i38;
            t[ind][1] = (ULO) i38dis;
            t[ind][2] = (mode == 0) ?
                           (reg*4 + (ULO) d):
                           (reg*4 + (ULO) a);
            t[ind][3] = (ULO) arw[flat];
            t[ind][4] = 12 + tarw[flat];
            }
       }   
}
/*
 Instruction MOVE.W to SR
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourcereg 3-sourceroutine
                4-cycle count                   
*/

ULO i39dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  sprintf(st,"$%.6X %.4X            (PRIV) MOVE.W  ",prc,opc);
  if (mode == 7) mode += reg;
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",SR");
  
  return prc;
}

void i39ini(void)
{
  ULO op=0x46c0,ind,reg,mode,flat;
    for (reg = 0; reg < 8; reg++) 
      for (mode = 0; mode < 8; mode++) {
        flat = (mode != 7) ? mode : (mode+reg);
        if (data[flat]) {
            ind = op|mode<<3|reg;
            t[ind][0] = (ULO) i39;
            t[ind][1] = (ULO) i39dis;
            t[ind][2] = (mode == 0) ?
                           (reg*4 + (ULO) d):
                           (reg*4 + (ULO) a);
            t[ind][3] = (ULO) arw[flat];
            t[ind][4] = 12 + tarw[flat];
            }
      }    
}
/*
 Instruction MOVE.W from SR
 Table-usage:

  0-Routinepointer 1-disasm routine  2-destreg 3-destroutine
                4-cycle count                   
*/

ULO i40dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  sprintf(st,"$%.6X %.4X                   MOVE.W  SR,",prc,opc);
  if (mode == 7) mode += reg;
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  
  return prc;
}

void i40ini(void)
{
  ULO op=0x40c0,ind,reg,mode,flat;
    for (reg = 0; reg < 8; reg++) 
      for (mode = 0; mode < 8; mode++) { 
        flat = (mode != 7) ? mode : (mode+reg);
        if (data[flat] & alterable[flat]) {
            ind = op|mode<<3|reg;
            t[ind][0] = (ULO) i40;
            t[ind][1] = (ULO) i40dis;
            t[ind][2] = (mode == 0) ?
                           (reg*4 + (ULO) d):
                           (reg*4 + (ULO) a);
            t[ind][3] = (ULO) aww[flat];
            t[ind][4] = (flat < 2) ? 6:(8 + tarw[flat]);
            }
     }     
}

/*
 Instruction MOVE.L USP to/from Ax
 Table-usage:

  0-Routinepointer 1-disasm routine  2-reg
*/

ULO i41dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,dir=opc&0x8;
  if (dir == 0)
    sprintf(st,"$%.6X %.4X            (PRIV) MOVE.L  A%1d,USP",prc,opc,reg);
  else
    sprintf(st,"$%.6X %.4X            (PRIV) MOVE.L  USP,A%1d",prc,opc,reg);
  
  return prc+2;
}

void i41ini(void)
{
  ULO op=0x4e60,ind,reg,dir;
  for (dir = 0; dir < 2; dir++)
    for (reg = 0; reg < 8; reg++) {
      ind = op|reg|dir<<3;
      t[ind][0] = (dir == 0) ?
                           (ULO) i41_2:
                           (ULO) i41_1;
      t[ind][1] = (ULO) i41dis;
      t[ind][2] = (ULO) a + reg*4;
      }
}

/*
 Instruction MOVEA.W/L <ea>,An
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Areg        5-cycle count                   
*/

ULO i42dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,dreg=(opc&DREG)>>9,o=opc&0x1000,mode=(opc&SMOD)>>3,size;
  size = (o == 0) ? 32:16;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   MOVEA.  ",prc,opc);
  st[37] = (size == 16) ?
              'W':
              'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,size,1,&pos);
  
  return prc;
}

void i42ini(void)
{
  ULO op=0x2040,ind,regd,modes,regs,flats,o;
  for (o = 0; o < 2; o++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) {
          flats = (modes != 7) ? modes : (modes+regs);
          if (allmodes[flats]) {
            ind = op|regd<<9|o<<12|modes<<3|regs;
            t[ind][0] = (o == 0) ?
                           (ULO) i42_L:
                           (ULO) i42_W;
            t[ind][1] = (ULO) i42dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO)((o == 0) ?
                           arl[flats]:
                           arw[flats]);
            t[ind][4] = regd*4 + (ULO) a;
            t[ind][5] = (o == 0) ?
                           (4 + tarl[flats]):
                           (4 + tarw[flats]);
            }
          }
}

/*
 Instruction MOVEM.W/L 
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-earead/write  4-base cycle time
*/

ULO i43dis(ULO prc,ULO opc,char *st) 
{
  char tmp[10];
  ULO i,first,pos=18,reg=opc&SREG,mode=(opc&SMOD)>>3,size,dir=opc&0x0400,regmask;
  ULO next=1;

  size = ((opc&BIT6) == 0) ? 16:32;
  regmask = fetw(prc+2);
  if (mode == 7) mode += reg;

  if (dir == 0 && mode == 4) {  /* Register to memory, predecrement */
    sprintf(st,"$%.6X %.4X %.4X              MOVEM.%s ",prc,opc,regmask,((size==16)?"W":"L"));
    first=TRUE;
    for (i=0;i<8;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"A%d",(7-i));
          strcat(st,tmp);
          first = FALSE;
          }
        else {
          sprintf(tmp,"%d",(7-i));
          strcat(st,tmp);
          }
        }
    for (i=8;i<16;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"D%d",(15-i));
          strcat(st,tmp);
          first = FALSE;
          next = FALSE;
          }
        else {
          if (next)
            sprintf(tmp,"D%d",(15-i));
          else
            sprintf(tmp,"%d",(15-i));
          next = FALSE;
          strcat(st,tmp);
          }
        }
      strcat(st,",");
      prc = disfordel(reg,prc+4,st,size,mode,&pos);
    }     
  else if (dir==0x0400) { /* Memory to register, any legal adressmode */
    sprintf(st,"$%.6X %.4X %.4X              MOVEM.%s ",prc,opc,regmask,((size==16)?"W":"L"));

    prc = disfordel(reg,prc+4,st,size,mode,&pos);
    strcat(st,",");
    first=TRUE;
    for (i=0;i<8;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"D%d",i);
          strcat(st,tmp);
          first = FALSE;
          }
        else {
          sprintf(tmp,"%d",i);
          strcat(st,tmp);
          }
        }
    for (i=8;i<16;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"A%d",i-8);
          strcat(st,tmp);
          first = FALSE;
          next = FALSE;
          }
        else {
          if (next)
            sprintf(tmp,"A%d",i-8);
          else
            sprintf(tmp,"%d",i-8);
          next = FALSE;  
          strcat(st,tmp);
          }
        }
    }     
  else {  /* Register to memory, the rest of the adr.modes */
    sprintf(st,"$%.6X %.4X %.4X              MOVEM.%s ",prc,opc,regmask,((size==16)?"W":"L"));
    first=TRUE;
    for (i=0;i<8;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"D%d",i);
          strcat(st,tmp);
          first = FALSE;
          }
        else {
          sprintf(tmp,"%d",i);
          strcat(st,tmp);
          }
        }
    for (i=8;i<16;i++) {
      if ((regmask&(1<<i)) != 0)
        if (first) {
          sprintf(tmp,"A%d",(i-8));
          strcat(st,tmp);
          first = FALSE;
          next = FALSE;
          }
        else {
          if (next)
            sprintf(tmp,"A%d",(i-8));
          else
            sprintf(tmp,"%d",(i-8));
          next = FALSE;
          strcat(st,tmp);
          }

        }
    strcat(st,",");
    prc = disfordel(reg,prc+4,st,size,mode,&pos);
    }     

  
  return prc;
}

void i43ini(void)
{
  ULO op=0x4880,ind,modes,regs,flats,dir,sz;
  for (dir = 0; dir < 2; dir++)
    for (sz = 0; sz < 2; sz++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) {
          flats = (modes != 7) ? modes : (modes+regs);
          if (control[flats] || ((flats == 3) && (dir == 1)) || ((flats == 4) && (dir == 0))) {
            ind = op|dir<<10|sz<<6|modes<<3|regs;
            t[ind][0] = (sz == 0) ?
                           ((dir == 0) ?
                             ((flats == 4) ?
                               (ULO) i43_W_PREM:
                               (ULO) i43_W_CONM):
                             ((flats == 3) ?
                               (ULO) i43_W_POSTR:
                               (ULO) i43_W_CONR)):
                           ((dir == 0) ?
                             ((flats == 4) ?
                               (ULO) i43_L_PREM:
                               (ULO) i43_L_CONM):
                             ((flats == 3) ?
                               (ULO) i43_L_POSTR:
                               (ULO) i43_L_CONR));
            t[ind][1] = (ULO) i43dis;
            t[ind][2] = regs*4 + (ULO) a;
            t[ind][3] = (ULO) eac[flats];
            t[ind][4] = (sz == 0) ?
                           (8 + tarw[flats]):
                           (8 + tarl[flats]);
            }
          }
}

/*
 Instruction MOVEP
*/

ULO i44dis(ULO prc,ULO opc,char *st) 
{

  ULO i,pos=13;
  sprintf(st,"$%.6X %.4X                   MOVEP",prc,opc);
  
  return prc+4;
}

void i44ini(void)
{
  ULO op=0x0008,ind,dreg,areg,mode;
  for (areg = 0; areg < 8; areg++) 
   for (dreg = 0; dreg < 8; dreg++) 
    for (mode = 4; mode < 8; mode++) {
      ind = op|areg|mode<<6|dreg<<9;
      if (mode == 4) t[ind][0] = (ULO) i44_W_1;
      else if (mode == 5) t[ind][0] = (ULO) i44_L_1;
      else if (mode == 6) t[ind][0] = (ULO) i44_W_2;
      else if (mode == 7) t[ind][0] = (ULO) i44_L_2;
      t[ind][1] = (ULO) i44dis;
      t[ind][2] = (ULO) a + areg;
      t[ind][3] = (ULO) d + dreg;
      }
}

/*
 Instruction MOVEQ.L #8 bit signed imm,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-reg 3-32 bit immediate
                   4-flag byte                          
*/

ULO i45dis(ULO prc,ULO opc,char *st) 
{

  ULO i,pos=13,reg=(opc&0xe00)>>9;
  sprintf(st,"$%.6X %.4X                   MOVEQ.L #$%8.8X,D%d",prc,opc,t[opc][3],reg);
  
  return prc+2;
}

void i45ini(void)
{
  ULO op=0x7000,ind,reg;
  int imm;
  for (reg = 0; reg < 8; reg++) 
    for (imm = -128; imm < 128; imm++) {
      ind = op|reg<<9|imm&0xff;
      t[ind][0] = (ULO) i45;
      t[ind][1] = (ULO) i45dis;
      t[ind][2] = reg*4 + (ULO) d;
      t[ind][3] = imm;
      t[ind][4] = 0 | ((imm < 0) ? 0x8:0) | ((imm == 0) ? 0x4:0);
      }
}

/*
 Instruction MULS.W <ea>,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg                          
*/

ULO i46dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   MULS.W  ",prc,opc);
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,16,0,&pos);
  
  return prc;
}

void i46ini(void)
{
  ULO op=0xc1c0,ind,regd,modes,regs,flats;
  for (regd = 0; regd < 8; regd++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (data[flats]) {
          ind = op|regd<<9|modes<<3|regs;
          t[ind][0] = (ULO) i46;
          t[ind][1] = (ULO) i46dis;
          t[ind][2] = regs*4 + ((modes == 0) ?
                                  (ULO) d:
                                  (ULO) a);
          t[ind][3] = (ULO) arw[flats];
          t[ind][4] = regd*4 + (ULO) d;
          }
       }
}

/*
 Instruction MULU.W <ea>,Dn
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg                          
*/

ULO i47dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   MULU.W  ",prc,opc);
  prc = disfordel(reg,prc+2,st,16,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,16,0,&pos);
  
  return prc;
}

void i47ini(void)
{
  ULO op=0xc0c0,ind,regd,modes,regs,flats;
  for (regd = 0; regd < 8; regd++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) 
        if (data[(flats = (modes != 7) ? modes : (modes+regs))]) {
          ind = op|regd<<9|modes<<3|regs;
          t[ind][0] = (ULO) i47;
          t[ind][1] = (ULO) i47dis;
          t[ind][2] = regs*4 + ((modes == 0) ?
                                  (ULO) d:
                                  (ULO) a);
          t[ind][3] = (ULO) arw[flats];
          t[ind][4] = regd*4 + (ULO) d;
          }
}

/*
 Instruction NBCD.B <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine 4-eawrite
                   5-cycle count                   
*/

ULO i48dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   NBCD.B  ",prc,opc);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}

void i48ini(void)
{
  ULO op=0x4800,ind,modes,regs,flats,size;
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|modes<<3|regs;
          t[ind][0] = (ULO) i48_B;
          t[ind][1] = (ULO) i48dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) parb[flats];
          t[ind][4] = (ULO) awb[flats];
          t[ind][5] = (modes == 0) ?
                               6:
                               (8+tarb[flats]);
          }
      }
}
/*
 Instruction NEG.X <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine 4-eawrite
                   5-cycle count                   
*/

ULO i49dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   NEG.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}

void i49ini(void)
{
  ULO op=0x4400,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
        t[ind][0] = (size == 0) ? (ULO) i49_B:
                      (size == 1) ? (ULO) i49_W:(ULO) i49_L;
        t[ind][1] = (ULO) i49dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO)((size == 0) ?
                       parb[flats]:
                       ((size == 1) ?
                          parw[flats]:
                          parl[flats]));
        t[ind][4] = (ULO)((size == 0) ?
                       awb[flats]:
                       ((size == 1) ?
                          aww[flats]:
                          awl[flats]));
        t[ind][5] = (size == 0) ?
                       (4+tarb[flats]):
                       ((size == 1) ?
                          (4+tarw[flats]):
                          ((modes == 0) ?
                             6:
                             (4+tarl[flats])));
        }
      }
}

/*
 Instruction NEGX.X <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine 4-eawrite
                   5-cycle count                   
*/

ULO i50dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   NEGX.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}

void i50ini(void)
{
  ULO op=0x4000,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
        t[ind][0] = (size == 0) ? (ULO) i50_B:
                      (size == 1) ? (ULO) i50_W:(ULO) i50_L;
        t[ind][1] = (ULO) i50dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO)((size == 0) ?
                       parb[flats]:
                       (size == 1) ?
                          parw[flats]:
                          parl[flats]);
        t[ind][4] = (ULO)((size == 0) ?
                       awb[flats]:
                       (size == 1) ?
                          aww[flats]:
                          awl[flats]);
        t[ind][5] = (size == 0) ?
                       (4+tarb[flats]):
                       (size == 1) ?
                          (4+tarw[flats]):
                          (modes == 0) ?
                             6:
                             (4+tarl[flats]);
        }
      }
}

/*
 Instruction NOP
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i51dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7;
  sprintf(st,"$%.6X %.4X                   NOP",prc,opc);
  
  return prc+2;
}

void i51ini(void)
{
  ULO op=0x4e71;
  t[op][0] = (ULO) i51;
  t[op][1] = (ULO) i51dis;
}

/*
 Instruction NOT.X <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine 4-eawrite
                   5-cycle count                   
*/

ULO i52dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   NOT.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}

void i52ini(void)
{
  ULO op=0x4600,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
        t[ind][0] = (size == 0) ? (ULO) i52_B:
                      (size == 1) ? (ULO) i52_W:(ULO) i52_L;
        t[ind][1] = (ULO) i52dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO)((size == 0) ?
                       parb[flats]:
                       (size == 1) ?
                          parw[flats]:
                          parl[flats]);
        t[ind][4] = (ULO)((size == 0) ?
                       awb[flats]:
                       (size == 1) ?
                          aww[flats]:
                          awl[flats]);
        t[ind][5] = (size == 0) ?
                       (4+tarb[flats]):
                       (size == 1) ?
                          (4+tarw[flats]):
                          (modes == 0) ?
                             6:
                             (4+tarl[flats]);
        }
      }
}


/*
 Instruction OR.X <ea>,Dn / OR.X Dn,<ea>
 Table-usage:
 Type 1:  OR <ea>,Dn

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5-cycle count                   
 Type 2:  OR Dn,<ea>

  0-Routinepointer 1-disasm routine  2-<ea>reg 3-get <ea> routine
                   4-Dreg        5-cycle count   6-write<ea> routine                
*/

ULO i53dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,o=opc&0x100,mode=(opc&0x38)>>3,
      size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   OR.     ",prc,opc);
  st[34] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = (o == 0) ? disfordel(reg,prc+2,st,size,mode,&pos) :
                   disfordel(dreg,prc+2,st,size,0,&pos);
  strcat(st,",");
  prc = (o != 0) ? disfordel(reg,prc,st,size,mode,&pos) :
                   disfordel(dreg,prc,st,size,0,&pos);
  
  return prc;
}

void i53ini(void)
{
  ULO op=0x8000,ind,regd,modes,regs,flats,size,o;
  for (o = 0; o < 2; o++)
    for (size = 0; size < 3; size++)
      for (regd = 0; regd < 8; regd++) 
        for (modes = 0; modes < 8; modes++) 
          for (regs = 0; regs < 8; regs++) {
            flats = (modes != 7) ?
                       modes:
                       (modes+regs);
            if ((o==0 && data[flats]) ||
                (o==1 && alterable[flats] && memory[flats])) {
              ind = op|regd<<9|o<<8|size<<6|modes<<3|regs;
              t[ind][0] = (o == 0) ?
                             (size == 0) ?
                                (ULO) i53_B_1:
                                (size == 1) ?
                                   (ULO) i53_W_1:
                                   (ULO) i53_L_1:
                             (size == 0) ?
                                (ULO) i53_B_2:
                                (size == 1) ?
                                   (ULO) i53_W_2:
                                   (ULO) i53_L_2;
              t[ind][1] = (ULO) i53dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO)((o == 0) ?
                             (size == 0) ?
                                arb[flats]:
                                (size == 1) ?
                                   arw[flats]:
                                   arl[flats]:
                             (size == 0) ?
                                parb[flats]:
                                (size == 1) ?
                                   parw[flats]:
                                   parl[flats]);
              t[ind][4] = regd*4 + (ULO) d;
              t[ind][5] = (o == 0) ?
                             (size == 0) ?
                                (4+tarb[flats]):
                                (size == 1) ?
                                   (4+tarw[flats]):
                                   (flats <= 1 || flats == 11) ?
                                      (8+tarl[flats]):
                                      (6+tarl[flats]):
                             (size == 0) ?
                                (8 + tarb[flats]):
                                (size == 1) ?
                                   (8 + tarw[flats]):
                                   (12 + tarl[flats]);
              if (o == 1) t[ind][6] = (ULO)((size == 0) ?
                                         awb[flats]:
                                         (size == 1) ?
                                            aww[flats]:
                                            awl[flats]);
              }
            }
}



/*
 Instruction ORI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine               
*/


ULO i54dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  if (mode == 11 && size == 16)
    sprintf(st,"$%.6X %.4X            (PRIV) ORI.    ",prc,opc);
  else
    sprintf(st,"$%.6X %.4X                   ORI.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  if (mode == 11)
    strcat(st,(size == 8) ? "CCR":"SR");
  else
    prc = disfordel(reg,prc,st,size,mode,&pos);
  
  return prc;
}

void i54ini(void)
{
  ULO op=0x0000,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if ((data[flats] && alterable[flats]) || (flats == 11 && size < 2)) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         ((flats == 11) ?
                           (ULO) i54_B_CCR:
                           (ULO) i54_B):
                         (size == 1) ?
                            ((flats == 11) ?
                              (ULO) i54_W_SR:
                              (ULO) i54_W):
                            (ULO) i54_L; 
          t[ind][1] = (ULO) i54dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO)((size == 0) ?
                         parb[flats]:
                         (size == 1) ?
                            parw[flats]:
                            parl[flats]);
          t[ind][4] = (flats == 11) ?
                        20:
                        (size == 0) ?
                         (flats == 0) ?
                            8:
                            (12 + tarb[flats]):
                         (size == 1)  ?
                            (flats == 0) ?
                               8:
                               (12 + tarw[flats]):
                            (flats == 0) ?
                               16:
                               (20 + tarl[flats]);
          t[ind][5] = (ULO)((size == 0) ?
                         awb[flats]:
                         (size == 1) ? 
                            aww[flats]:
                            awl[flats]);
          }
        }
}

/*
 Instruction pea <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-earead
                   4-cycle count                   
*/

ULO i55dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,areg=(opc&DREG)>>9,reg=opc&SREG,mode=(opc&SMOD)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   PEA.L   ",prc,opc);
  prc = disfordel(reg,prc+2,st,32,mode,&pos);
  
  return prc;
}

void i55ini(void)
{
  ULO op=0x4840,ind,modes,regs,flats;
  for (modes = 0; modes < 8; modes++)
    for (regs = 0; regs < 8; regs++) {
      flats = (modes != 7) ? modes : (modes+regs);
      if (control[flats]) {
        ind = op|modes<<3|regs;
        t[ind][0] = (ULO) i55;
        t[ind][1] = (ULO) i55dis;
        t[ind][2] = regs*4 + (ULO) a;
        t[ind][3] = (ULO) eac[flats];
        t[ind][4] = (flats == 2) ? 12:
                      (flats == 5) ? 16:
                        (flats == 6) ? 20:
                          (flats == 7) ? 16:
                            (flats == 9) ? 20:
                              (flats == 10) ? 16:20;
        }
      }
}

/*
 Instruction RESET
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i56dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7;
  sprintf(st,"$%.6X %.4X                   RESET",prc,opc);
  
  return prc+2;
}

void i56ini(void)
{
  ULO op=0x4e70;
  t[op][0] = (ULO) i56;
  t[op][1] = (ULO) i56dis;
}

/*
 Instruction ROL/ROR.X #i,<ea>
 3 versions: 
 Table-usage:

 ROX.X Dx,Dy
  0-Routinepointer 1-disasm routine  2-shcountreg 3-dreg
                4-cycle count               

 RoX.X #,Dy
  0-Routinepointer 1-disasm routine  2-shiftcount 3-dreg
                4-cycle count               

 RoX.X #1,<ea>
  0-Routinepointer 1-disasm routine  2-eareg 3-earead
                4-eawrite        5-cycle count               
*/


ULO i57dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=((opc&SIZE) == 0xc0) ?
                                                      64:size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  if (size == 64) {
    sprintf(st,"$%.6X %.4X                   RO .W   #$1,",prc,opc);
    prc = disfordel(reg,prc+2,st,16,mode,&pos);
    }
  else {
    if ((opc&BIT5) == 0)
      sprintf(st,"$%.6X %.4X                   RO .    #$%1X,D%1d",prc,opc,t[opc][2],(t[opc][3]-(ULO)d)>>2);
    else  
      sprintf(st,"$%.6X %.4X                   RO .    D%1d,D%1d",prc,opc,(t[opc][2]-(ULO) d)>>2,(t[opc][3]-(ULO)d)>>2);
    st[35] = (size == 8) ?
                'B':
                (size == 16) ?
                   'W':
                   'L';
    prc += 2;
    }
  st[33] = ((opc&BIT8) == 0) ? 'R':'L';
  
  return prc;
}

void i57ini(void)
{
  ULO op=0xe018,ind,dir,modes,regs,regc,flats,size,immreg;
  for (size = 0; size < 3; size++)
    for (regc = 0; regc < 8; regc++) 
      for (dir = 0; dir < 2; dir++) 
        for (regs = 0; regs < 8; regs++)
          for (immreg = 0; immreg < 2; immreg++) {
            ind = op|regc<<9|dir<<8|size<<6|immreg<<5|regs;
            t[ind][0] = (size == 0) ?
                           ((immreg==0) ?
                             ((dir==0) ?
                               (ULO) i57_RI_I_B:
                               (ULO) i57_LE_I_B):
                             ((dir==0) ?
                               (ULO) i57_RI_R_B:
                               (ULO) i57_LE_R_B)):
                           ((size == 1) ?
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i57_RI_I_W:
                                 (ULO) i57_LE_I_W):
                               ((dir==0) ?
                                 (ULO) i57_RI_R_W:
                                 (ULO) i57_LE_R_W)):
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i57_RI_I_L:
                                 (ULO) i57_LE_I_L):
                               ((dir==0) ?
                                 (ULO) i57_RI_R_L:
                                 (ULO) i57_LE_R_L)));
            t[ind][1] = (ULO) i57dis;
            t[ind][2] = (immreg == 0) ?
                          ((regc == 0) ?
                            8:
                            regc):
                          (regc*4 + (ULO) d);
            t[ind][3] = regs*4 + (ULO) d;
            t[ind][4] = (size == 3) ?
                          ((immreg == 0) ?
                            (8+(t[ind][2]*2)):
                            8):
                          ((immreg == 0) ?
                            (6+(t[ind][2]*2)):
                            6);
            }


  op = 0xe6c0;
  for (dir = 0; dir < 2; dir++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (memory[flats] && alterable[flats]) {
          ind = op|dir<<8|modes<<3|regs;
          t[ind][0] = (dir == 0) ?
                        (ULO) i57_RI_M_W:
                        (ULO) i57_LE_M_W;
          t[ind][1] = (ULO) i57dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) parw[flats];
          t[ind][4] = (ULO) aww[flats];
          t[ind][5] = 8+tarw[flats];
          }
        }
}

/*
 Instruction ROXL/ROXR.X #i,<ea>
 3 versions: 
 Table-usage:

 ROXX.X Dx,Dy
  0-Routinepointer 1-disasm routine  2-shcountreg 3-dreg
                4-cycle count               

 ROXX.X #,Dy
  0-Routinepointer 1-disasm routine  2-shiftcount 3-dreg
                4-cycle count               

 ROXX.X #1,<ea>
  0-Routinepointer 1-disasm routine  2-eareg 3-earead
                4-eawrite        5-cycle count               
*/


ULO i59dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&SREG,mode=(opc&SMOD)>>3,size=((opc&SIZE) == 0xc0) ?
                                                      64:size000110(opc&SIZE);
  if (mode == 7) mode += reg;
  if (size == 64) {
    sprintf(st,"$%.6X %.4X                   ROX .W  #$1,",prc,opc);
    prc = disfordel(reg,prc+2,st,16,mode,&pos);
    }
  else {
    if ((opc&BIT5) == 0)
      sprintf(st,"$%.6X %.4X                   ROX .   #$%1X,D%1d",prc,opc,t[opc][2],(t[opc][3]-(ULO)d)>>2);
    else  
      sprintf(st,"$%.6X %.4X                   ROX .   D%1d,D%1d",prc,opc,(t[opc][2]-(ULO)d)>>2,(t[opc][3]-(ULO)d)>>2);
    st[36] = (size == 8) ?
                'B':
                (size == 16) ?
                   'W':
                   'L';
    prc += 2;
    }
  st[34] = ((opc&BIT8) == 0) ? 'R':'L';
  
  return prc;
}

void i59ini(void)
{
  ULO op=0xe010,ind,dir,modes,regs,regc,flats,size,immreg;
  for (size = 0; size < 3; size++)
    for (regc = 0; regc < 8; regc++) 
      for (dir = 0; dir < 2; dir++) 
        for (regs = 0; regs < 8; regs++)
          for (immreg = 0; immreg < 2; immreg++) {
            ind = op|regc<<9|dir<<8|size<<6|immreg<<5|regs;
            t[ind][0] = (size == 0) ?
                           ((immreg==0) ?
                             ((dir==0) ?
                               (ULO) i59_RI_I_B:
                               (ULO) i59_LE_I_B):
                             ((dir==0) ?
                               (ULO) i59_RI_R_B:
                               (ULO) i59_LE_R_B)):
                           ((size == 1) ?
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i59_RI_I_W:
                                 (ULO) i59_LE_I_W):
                               ((dir==0) ?
                                 (ULO) i59_RI_R_W:
                                 (ULO) i59_LE_R_W)):
                             ((immreg==0) ?
                               ((dir==0) ?
                                 (ULO) i59_RI_I_L:
                                 (ULO) i59_LE_I_L):
                               ((dir==0) ?
                                 (ULO) i59_RI_R_L:
                                 (ULO) i59_LE_R_L)));
            t[ind][1] = (ULO) i59dis;
            t[ind][2] = (immreg == 0) ?
                          ((regc == 0) ?
                            8:
                            regc):
                          (regc*4 + (ULO) d);
            t[ind][3] = regs*4 + (ULO) d;
            t[ind][4] = (size == 3) ?
                          ((immreg == 0) ?
                            (8+(t[ind][2]*2)):
                            8):
                          ((immreg == 0) ?
                            (6+(t[ind][2]*2)):
                            6);
            }


  op = 0xe4c0;
  for (dir = 0; dir < 2; dir++) 
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (memory[flats] && alterable[flats]) {
          ind = op|dir<<8|modes<<3|regs;
          t[ind][0] = (dir == 0) ?
                        (ULO) i59_RI_M_W:
                        (ULO) i59_LE_M_W;
          t[ind][1] = (ULO) i59dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO) parw[flats];
          t[ind][4] = (ULO) aww[flats];
          t[ind][5] = 8+tarw[flats];
          }
        }
}




/*
 Instruction RTE
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i61dis(ULO prc,ULO opc,char *st) 
{

  sprintf(st,"$%.6X %.4X                   RTE",prc,opc);
  
  return prc+2;
}

void i61ini(void)
{
  ULO op=0x4e73;
  t[op][0] = (ULO) i61;
  t[op][1] = (ULO) i61dis;
}

/*
 Instruction RTR
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i62dis(ULO prc,ULO opc,char *st) 
{

  sprintf(st,"$%.6X %.4X                   RTR",prc,opc);
  
  return prc+2;
}

void i62ini(void)
{
  ULO op=0x4e77;
  t[op][0] = (ULO) i62;
  t[op][1] = (ULO) i62dis;
}    

/*
 Instruction RTS
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i63dis(ULO prc,ULO opc,char *st) 
{

  sprintf(st,"$%.6X %.4X                   RTS",prc,opc);
  
  return prc+2;
}

void i63ini(void)
{
  ULO op=0x4e75;
  t[op][0] = (ULO) i63;
  t[op][1] = (ULO) i63dis;
}    

/*
 Instruction SBCD.B Ry,Rx / SBCD.B -(Ay),-(Ax)
 Table-usage: 0 - Routinepointer 1 - disasm routine 2 - Rx*4(dest) 3 - Ry*4
*/

ULO i64dis(ULO prc,ULO opc,char *st)
{

  sprintf(st,((opc&0x8 == 0) ?
                "$%.6X %.4X                   SBCD.B  D%1d,D%1d":
                "$%.6X %.4X                   SBCD.B  -(A%1d),-(A%1d)"),
                prc,opc,opc&0x3,(opc&0xe00)>>9);
  
  return prc+2;
}

void i64ini(void)
{
  ULO op=0x8100,ind,rm,rx,ry;
  for (rm = 0; rm <= 1; rm++) 
    for (rx = 0; rx <= 7; rx++) 
      for (ry = 0; ry <= 7; ry++) {
        ind = op|ry|rx<<9|rm<<3;
        t[ind][0] = (rm == 0) ?
                       (ULO) i64_1:
                       (ULO) i64_2;
        t[ind][1] = (ULO) i64dis;
        t[ind][2] = (rm == 0) ?
                       (rx*4 + (ULO) d):
                       (rx*4 + (ULO) a);
        t[ind][3] = (rm == 0) ?
                       (ry*4 + (ULO) d):
                       (ry*4 + (ULO) a);
        }
}

/*
 Instruction Scc.B <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-reg*4+d  3-ea write routine
                4-handle routine 5-cyclecount not taken  6-cycle count taken
*/

ULO i65dis(ULO prc,ULO opc,char *st)
{

  ULO pos=13,j,adr,mode=(opc&0x38)>>3,reg=(opc&0x7);
  sprintf(st,"$%.6X %.4X                   S",prc,opc);
  switch ((opc&0xf00)>>8) {
     case 0:  strcat(st,"T.B    ");
              break;
     case 1:  strcat(st,"F.B    ");
              break;           
     case 2:  strcat(st,"HI.B   ");
              break;
     case 3:  strcat(st,"LS.B   ");
              break;
     case 4:  strcat(st,"CC.B   ");
              break;
     case 5:  strcat(st,"CS.B   ");
              break;
     case 6:  strcat(st,"NE.B   ");
              break;
     case 7:  strcat(st,"EQ.B   ");
              break;
     case 8:  strcat(st,"VC.B   ");
              break;
     case 9:  strcat(st,"VS.B   ");
              break;
     case 10: strcat(st,"PL.B   ");
              break;
     case 11: strcat(st,"MI.B   ");
              break;
     case 12: strcat(st,"GE.B   ");
              break;
     case 13: strcat(st,"LT.B   ");
              break;
     case 14: strcat(st,"GT.B   ");
              break;
     case 15: strcat(st,"LE.B   ");
              break;
       }
  if (mode == 7) mode += reg;
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}


void i65ini(void)
{
  ULO op=0x50c0,c,ind,flat,mode,reg;
  ULO routines[16] = {(ULO) cc0,(ULO) cc1false,(ULO) cc2,(ULO) cc3,(ULO) cc4,(ULO) cc5,(ULO) cc6,(ULO) cc7,(ULO) cc8,(ULO) cc9,(ULO) cca,(ULO) ccb,(ULO) ccc,(ULO) ccd,(ULO) cce,(ULO) ccf};
  for (c = 0; c < 16; c++)
    for (mode = 0; mode < 8; mode++) 
      for (reg = 0; reg < 8; reg++) {
        flat = (mode != 7) ?
                   mode:
                   (mode+reg);
        if (data[flat] && alterable[flat]) {
          ind = op|c<<8|mode<<3|reg;
          t[ind][0] = (ULO) i65;
          t[ind][1] = (ULO) i65dis;
          t[ind][2] = (mode == 0) ? (reg*4 + (ULO) d):
                                    (reg*4 + (ULO) a);  
          t[ind][3] = (ULO) awb[flat];
          t[ind][4] = routines[c];
          t[ind][5] = 4+tarb[flat];
          t[ind][6] = (flat == 0) ?
                         6:
                         (4+tarb[flat]);
          }
        }
}

/*
 Instruction STOP #
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i66dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7;
  sprintf(st,"$%.6X %.4X                   STOP #$%.4X",prc,opc,fetw(prc+2));
  
  return prc+4;
}

void i66ini(void)
{
  ULO op=0x4e72;
  t[op][0] = (ULO) i66;
  t[op][1] = (ULO) i66dis;
}

/*
 Instruction SUB.X <ea>,Dn / SUB.X Dn,<ea>
 Table-usage:
 Type 1:  SUB <ea>,Dn

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Dreg        5-cycle count                   
 Type 2:  SUB Dn,<ea>

  0-Routinepointer 1-disasm routine  2-<ea>reg 3-get <ea> routine
                   4-Dreg        5-cycle count   6-write<ea> routine                
*/

ULO i67dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,o=opc&0x100,mode=(opc&0x38)>>3,
      size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   SUB.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = (o == 0) ? disfordel(reg,prc+2,st,size,mode,&pos) :
                   disfordel(dreg,prc+2,st,size,0,&pos);
  strcat(st,",");
  prc = (o != 0) ? disfordel(reg,prc,st,size,mode,&pos) :
                   disfordel(dreg,prc,st,size,0,&pos);
  
  return prc;
}

void i67ini(void)
{
  ULO op=0x9000,ind,regd,modes,regs,flats,size,o;
  for (o = 0; o < 2; o++)
    for (size = 0; size < 3; size++)
      for (regd = 0; regd < 8; regd++) 
        for (modes = 0; modes < 8; modes++) 
          for (regs = 0; regs < 8; regs++) {
            flats = (modes != 7) ?
                       modes:
                       (modes+regs);
            if ((o==0 && size == 0 && data[flats]) ||
                (o==0 && size != 0 && allmodes[flats]) ||
                (o==1 && alterable[flats] && memory[flats])) {
              ind = op|regd<<9|o<<8|size<<6|modes<<3|regs;
              t[ind][0] = (o == 0) ?
                             (size == 0) ?
                                (ULO) i67_B_1:
                                (size == 1) ?
                                   (ULO) i67_W_1:
                                   (ULO) i67_L_1:
                             (size == 0) ?
                                (ULO) i67_B_2:
                                (size == 1) ?
                                   (ULO) i67_W_2:
                                   (ULO) i67_L_2;
              t[ind][1] = (ULO) i67dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO)((o == 0) ?
                             (size == 0) ?
                                arb[flats]:
                                (size == 1) ?
                                   arw[flats]:
                                   arl[flats]:
                             (size == 0) ?
                                parb[flats]:
                                (size == 1) ?
                                   parw[flats]:
                                   parl[flats]);
              t[ind][4] = regd*4 + (ULO) d;
              t[ind][5] = (o == 0) ?
                             (size == 0) ?
                                (4+tarb[flats]):
                                (size == 1) ?
                                   (4+tarw[flats]):
                                   (flats <= 1 || flats == 11) ?
                                      (8+tarl[flats]):
                                      (6+tarl[flats]):
                             (size == 0) ?
                                (8 + tarb[flats]):
                                (size == 1) ?
                                   (8 + tarw[flats]):
                                   (12 + tarl[flats]);
              if (o == 1) t[ind][6] = (ULO) ((size == 0) ?
                                         awb[flats]:
                                         (size == 1) ?
                                            aww[flats]:
                                            awl[flats]);
              }
            }
}


/*
 Instruction SUBA.W/L <ea>,An
 Table-usage:

  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-Areg        5-cycle count                   
*/



ULO i68dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,dreg=(opc&0xe00)>>9,o=opc&0x100,mode=(opc&0x38)>>3,size;
  size = (o == 0) ?
            16:
            32;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   SUBA.   ",prc,opc);
  st[36] = (size == 16) ?
              'W':
              'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  strcat(st,",");
  disfordel(dreg,prc,st,size,1,&pos);
  
  return prc;
}

void i68ini(void)
{
  ULO op=0x90c0,ind,regd,modes,regs,flats,o;
  for (o = 0; o < 2; o++)
    for (regd = 0; regd < 8; regd++) 
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) 
          if (allmodes[(flats = (modes != 7) ? modes : (modes+regs))]) {
            ind = op|regd<<9|o<<8|modes<<3|regs;
            t[ind][0] = (o == 0) ?
                           (ULO) i68_W:
                           (ULO) i68_L;
            t[ind][1] = (ULO) i68dis;
            t[ind][2] = (modes == 0) ?
                           (regs*4 + (ULO) d):
                           (regs*4 + (ULO) a);
            t[ind][3] = (ULO)((o == 0) ?
                           arw[flats]:
                           arl[flats]);
            t[ind][4] = regd*4 + (ULO) a;
            t[ind][5] = (o==0) ?
                           (8 + tarw[flats]):
                           (flats <= 1 || flats == 11) ?
                              (8 + tarl[flats]):
                              (6 + tarl[flats]);
            }
}


/*
 Instruction SUBI.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine               
*/

ULO i69dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   SUBI.   ",prc,opc);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(4,prc+2,st,size,11,&pos);
  strcat(st,",");
  prc = disfordel(reg,prc,st,size,mode,&pos);
  
  return prc;
}

void i69ini(void)
{
  ULO op=0x0400,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ? modes : (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
          t[ind][0] = (size == 0) ?
                         (ULO) i69_B:
                         (size == 1) ?
                            (ULO) i69_W:
                            (ULO) i69_L; 
          t[ind][1] = (ULO) i69dis;
          t[ind][2] = (modes == 0) ?
                         (regs*4 + (ULO) d):
                         (regs*4 + (ULO) a);
          t[ind][3] = (ULO)((size == 0) ?
                         parb[flats]:
                         (size == 1) ?
                            parw[flats]:
                            parl[flats]);
          t[ind][4] = (size == 0) ?
                         (flats == 0) ?
                            8:
                            (12 + tarb[flats]):
                         (size == 1)  ?
                            (flats == 0) ?
                               8:
                               (12 + tarw[flats]):
                            (flats == 0) ?
                               16:
                               (20 + tarl[flats]);
          t[ind][5] = (ULO)((size == 0) ?
                         awb[flats]:
                         (size == 1) ? 
                            aww[flats]:
                            awl[flats]);
          }
        }
}

/*
 Instruction SUBQ.X #i,<ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-cycle count    5-ea write routine 6-SUB immediate              
*/

ULO i70dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   SUBQ.   #$%.1d,",prc,opc,t[opc][6]);
  st[36] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}


void i70ini(void)
{
  ULO op=0x5100,ind,imm,modes,regs,flats,size,o;
    for (size = 0; size < 3; size++)
      for (modes = 0; modes < 8; modes++) 
        for (regs = 0; regs < 8; regs++) 
          for (imm = 0; imm < 8; imm++) {
            flats = (modes != 7) ?
                       modes:
                       (modes+regs);
            if ((size == 0 && alterable[flats] && modes != 1) ||
                (size != 0 && alterable[flats])) {
              ind = op|imm<<9|size<<6|modes<<3|regs;
              t[ind][0] = (size == 0) ?
                             (ULO) i70_B:
                             (size == 1) ?
                                ((modes == 1) ? (ULO) i70_ADR:(ULO) i70_W):
                                ((modes == 1) ? (ULO) i70_ADR:(ULO) i70_L);
              t[ind][1] = (ULO) i70dis;
              t[ind][2] = (modes == 0) ?
                             (regs*4 + (ULO) d):
                             (regs*4 + (ULO) a);
              t[ind][3] = (ULO)((size == 0) ?
                             parb[flats]:
                             (size == 1) ?
                                parw[flats]:
                                parl[flats]);
              t[ind][4] = (size == 0) ?
                             (flats == 0) ?
                                (4+tarb[flats]):
                                (8+tarb[flats]):
                             (size == 1) ?
                                (flats == 0) ?
                                   (4+tarw[flats]):
                                   (8+tarw[flats]):
                                (flats < 2) ?
                                   (8+tarl[flats]):
                                   (12+tarl[flats]);
              t[ind][5] = (ULO)((size == 0) ?
                             awb[flats]:
                             (size == 1) ?
                                aww[flats]:
                                awl[flats]);
              t[ind][6] = (imm == 0) ?
                             8:
                             imm;
              }
            }
}

/*
 Instruction SUB.X Ry,Rx / SUB.X -(Ay),-(Ax)
 Table-usage: 0 - Routinepointer 1 - disasm routine 2 - Rx*4(dest) 3 - Ry*4
*/

ULO i71dis(ULO prc,ULO opc,char *st)
{

  ULO size=(opc&0x00c0)>>6;
  sprintf(st,(((opc&0x8) == 0) ?
                "$%.6X %.4X                   SUBX.%s  D%d,D%d":
                "$%.6X %.4X                   SUBX.%s  -(A%d),-(A%d)"),
                prc,opc,(size == 0) ? "B":((size == 1) ? "W":"L"),(opc&SREG),(opc&DREG)>>9);
  
  return prc+2;
}


void i71ini(void)
{
  ULO op=0x9100,ind,rm,rx,ry,sz;
  for (sz = 0; sz <= 2; sz++)
    for (rm = 0; rm <= 1; rm++)
      for (rx = 0; rx <= 7; rx++) 
        for (ry = 0; ry <= 7; ry++) {
          ind = op|ry|rx<<9|rm<<3|sz<<6;
          t[ind][0] = (rm == 0) ?
                         (sz == 0) ?
                            (ULO) i71_B_1:
                            (sz == 1) ?
                               (ULO) i71_W_1:
                               (ULO) i71_L_1:
                         (sz == 0) ?
                            (ULO) i71_B_2:
                            (sz == 1) ?
                               (ULO) i71_W_2:
                               (ULO) i71_L_2;
          t[ind][1] = (ULO) i71dis;
          t[ind][2] = (rm == 0) ?
                         (rx*4 + (ULO) d):
                         (rx*4 + (ULO) a);
          t[ind][3] = (rm == 0) ?
                         (ry*4 + (ULO) d):
                         (ry*4 + (ULO) a);
          }
}

/*
 Instruction SWAP.W Dn
 Table-usage:
     0-Routinepointer 1-disasm routine  2-reg
*/

ULO i72dis(ULO prc,ULO opc,char *st) 
{

  sprintf(st,"$%.6X %.4X                   SWAP.W  D%1X",prc,opc,opc&0x7);
  
  return prc+2;
}

void i72ini(void)
{
  ULO op=0x4840,ind,reg;
  for (reg = 0; reg < 8; reg++) {
    ind = op|reg;
    t[ind][0] = (ULO) i72;
    t[ind][1] = (ULO) i72dis;
    t[ind][2] = reg*4 + (ULO) d;
    }    
}

/*
 Instruction TAS.B <ea>
 Table-usage:

  0-Routinepointer 1-disasm routine  2-eareg 3-ea read routine
                4-ea write routine  5-cycle count             
*/

ULO i73dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   TAS.B   ",prc,opc);
  prc = disfordel(reg,prc+2,st,8,mode,&pos);
  
  return prc;
}

void i73ini(void)
{
  ULO op=0x4ac0,ind,modes,regs,flats;
  for (modes = 0; modes < 8; modes++) 
    for (regs = 0; regs < 8; regs++) {
      flats = (modes != 7) ? modes : (modes+regs);
      if (data[flats] && alterable[flats]) {
        ind = op|modes<<3|regs;
        t[ind][0] = (ULO) i73;
        t[ind][1] = (ULO) i73dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO) parb[flats];
        t[ind][4] = (ULO) awb[flats];
        t[ind][5] = (flats == 0) ?
                       4:
                       (10 + tarb[flats]);
        }
      }
}

/*
 Instruction TRAP #
 Table-usage:
     0-Routinepointer 1-disasm routine  2-vector*4+80h
*/

ULO i74dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13;
  sprintf(st,"$%.6X %.4X                   TRAP    #$%1X",prc,opc,opc&0xf);
  
  return prc+2;
}

void i74ini(void)
{
  ULO op=0x4e40,ind,vektor;
  for (vektor = 0; vektor < 16; vektor++) {
    ind = op|vektor;
    t[ind][0] = (ULO) i74;
    t[ind][1] = (ULO) i74dis;
    t[ind][2] = vektor*4 + 0x80;
    }    
}

/*
 Instruction TRAPV
 Table-usage:
     0-Routinepointer 1-disasm routine
*/

ULO i75dis(ULO prc,ULO opc,char *st) 
{

  sprintf(st,"$%.6X %.4X                   TRAPV",prc,opc);
  
  return prc+2;
}

void i75ini(void)
{
  ULO op=0x4e76;
  t[op][0] = (ULO) i75;
  t[op][1] = (ULO) i75dis;
}

/*
 Instruction TST.X <ea>
 Table-usage:
  0-Routinepointer 1-disasm routine  2-sourreg 3-sourceroutine
                   4-cycle count                   
*/

ULO i76dis(ULO prc,ULO opc,char *st) 
{

  ULO pos=13,reg=opc&0x7,mode=(opc&0x38)>>3,size=(1+((opc&0xc0)>>6))<<3;
  if (mode == 7) mode += reg;
  sprintf(st,"$%.6X %.4X                   TST.    ",prc,opc);
  st[35] = (size == 8) ?
              'B':
              (size == 16) ?
                 'W':
                 'L';
  prc = disfordel(reg,prc+2,st,size,mode,&pos);
  
  return prc;
}

void i76ini(void)
{
  ULO op=0x4a00,ind,modes,regs,flats,size;
  for (size = 0; size < 3; size++)
    for (modes = 0; modes < 8; modes++) 
      for (regs = 0; regs < 8; regs++) {
        flats = (modes != 7) ?
                   modes:
                   (modes+regs);
        if (data[flats] && alterable[flats]) {
          ind = op|size<<6|modes<<3|regs;
        t[ind][0] = (size == 0) ?
                       (ULO) i76_B:
                       (size == 1) ?
                          (ULO) i76_W:
                          (ULO) i76_L;
        t[ind][1] = (ULO) i76dis;
        t[ind][2] = (modes == 0) ?
                       (regs*4 + (ULO) d):
                       (regs*4 + (ULO) a);
        t[ind][3] = (ULO)((size == 0) ?
                       arb[flats]:
                       (size == 1) ?
                          arw[flats]:
                          arl[flats]);
        t[ind][4] = (size == 0) ?
                       (4+tarb[flats]):
                       (size == 1) ?
                          (4+tarw[flats]):
                          (4+tarl[flats]);
        }
      }
}

/*
 Instruction UNLK An
 Table-usage:
     0-Routinepointer 1-disasm routine  2-reg
*/

ULO i77dis(ULO prc,ULO opc,char *st) 
{

  ULO reg=opc&0x7;
  sprintf(st,"$%.6X %.4X                   UNLK    A%1d",prc,opc,reg);
  
  return prc+2;
}

void i77ini(void)
{
  ULO op=0x4e58,ind,reg;
  for (reg = 0; reg < 8; reg++) {
    ind = op|reg;
    t[ind][0] = (ULO) i77;
    t[ind][1] = (ULO) i77dis;
    t[ind][2] = reg*4 + (ULO) a;
    }    
}


/* Set adressmode data to 68000 stubs */

void init_adrmode_tables_68000(void) {
        int i;

        for (i = 0; i < 12; i++) {
                arb[i] = arb68000[i];
                arw[i] = arw68000[i];
                arl[i] = arl68000[i];
                parb[i] = parb68000[i];
                parw[i] = parw68000[i];
                parl[i] = parl68000[i];
                awb[i] = awb68000[i];
                aww[i] = aww68000[i];
                awl[i] = awl68000[i];
                eac[i] = eac68000[i];
                tarb[i] = tarb68000[i];
                tarw[i] = tarw68000[i];
                tarl[i] = tarl68000[i];
                }
}

/*
   createstatustab
   Will translate the flags C X V N Z som the intel flag register
   X is set to the same as C since Intel doesn't have X
*/

void createstatustab(void) {
        ULO i;

        for (i = 0; i < 4096; i++)
        statustab[i]=(i&0x1)|((i&0x1)<<4)|((i&0xc0)>>4)|((i&0x800)>>10);
}


/* Make muls and mulu tables for clock calculations */

void makemultab(void) {
        ULO i,j,k;

        for (i = 0; i < 256; i++) {
                j = 0;
                for (k = 0; k < 8; k++) if (((i>>k) & 1) == 1) j++;
                mulutab[i] = j;

                j = 0;
                for (k = 0; k < 8; k++) if ((((i>>k) & 3) == 1) || (((i>>k) & 3) == 2)) j++; 
                mulstab[i] = j;
                }
}

void clear_opcode_table(void) {
        int i,j;

        for (i = 0; i <= 65535; i++) {
                t[i][0] = (ULO) &i00;
                t[i][1] = (ULO) &i00dis;
                for (j = 2; j <= 7; j++) t[i][j] = 0;
                }
}

void cpureset68000(void) {
        int i;

        for (i = 0; i < 8; i++) d[i] = a[i] = 0;
        usp = a[7] = config_memory_chipsize;
        ssp = config_memory_chipsize - 0x2000;
}

/* cpureset will clear all registers and set things up in a default way */

void cpureset(void) {
        int i;

        if (config_cpu_type == 3) cpureset68030();
        else cpureset68000();
}


/* cpuinit is called whenever the opcode tables needs to be changes */
/* 1. At emulator startup or */
/* 2. Cpu type has changed */

void cpuinit() {
        clear_opcode_table();

        if (config_cpu_type == 0) init_adrmode_tables_68000();
        else init_adrmode_tables_68030();

        makebcdtabs();
        createstatustab();

        i01ini();
        i02ini();
        i03ini();
        i04ini();
        i05ini();
        i06ini();
        i07ini();
        i08ini();
        i09ini();
        i11ini();
        i12ini();
        i13ini();
        i15ini();
        i17ini();
        i18ini();
        i19ini();
        i20ini();
        i21ini();
        i22ini();
        i23ini();
        i24ini();
        i25ini();
        i26ini();
        i27ini();
        i28ini();
        i29ini();
        i30ini();
        i31ini();
        i32ini();
        i33ini();
        i34ini();
        i35ini();
        i37ini();
        i38ini();
        i39ini();
        i40ini();
        i41ini();
        i42ini();
        i43ini();
        i44ini();
        i45ini();
        i46ini();
        i47ini();
        i48ini();
        i49ini();
        i50ini();
        i51ini();
        i52ini();
        i53ini();
        i54ini();
        i55ini();
        i56ini();
        i57ini();
        i59ini();
        i61ini();
        i62ini();
        i63ini();
        i64ini();
        i65ini();
        i66ini();
        i67ini();
        i68ini();
        i69ini();
        i70ini();
        i71ini();
        i72ini();
        i73ini();
        i74ini();
        i75ini();
        i76ini();
        i77ini();
        makemultab();
        if (config_cpu_type == 3) cpuinit68030();
}



